import datetime
import decimal
from collections.abc import Iterable, Mapping
from typing_extensions import TypeAlias

integer_types = int
text_type = str
binary_type = bytes

_XMLValue: TypeAlias = (
    str
    | bytes
    | int
    | bool
    | decimal.Decimal
    | Iterable[_XMLValue]
    | Mapping[str, _XMLValue]
    | datetime.datetime
    | datetime.date
    | None
)
_XML: TypeAlias = Mapping[str, _XMLValue]

class Generator:
    dict: _XML
    def __init__(self, dict: _XML) -> None: ...
    def generate(self) -> str: ...



import datetime
from typing import ClassVar

from ..core import WesternCalendar

class Switzerland(WesternCalendar):
    include_berchtolds_day: ClassVar[bool]
    include_st_josephs_day: ClassVar[bool]
    def has_berchtolds_day(self, year: int) -> bool: ...
    def get_federal_thanksgiving_monday(self, year: int) -> tuple[datetime.date, str]: ...

class Aargau(Switzerland): ...
class AppenzellInnerrhoden(Switzerland): ...
class AppenzellAusserrhoden(Switzerland): ...
class Bern(Switzerland): ...
class BaselLandschaft(Switzerland): ...
class BaselStadt(Switzerland): ...
class Fribourg(Switzerland): ...

class Geneva(Switzerland):
    def get_genevan_fast(self, year: int) -> tuple[datetime.date, str]: ...

class Glarus(Switzerland): ...
class Graubunden(Switzerland): ...
class Jura(Switzerland): ...
class Luzern(Switzerland): ...
class Neuchatel(Switzerland): ...
class Nidwalden(Switzerland): ...
class Obwalden(Switzerland): ...
class StGallen(Switzerland): ...
class Schaffhausen(Switzerland): ...
class Solothurn(Switzerland): ...
class Schwyz(Switzerland): ...
class Thurgau(Switzerland): ...
class Ticino(Switzerland): ...
class Uri(Switzerland): ...
class Vaud(Switzerland): ...
class Valais(Switzerland): ...
class Zug(Switzerland): ...
class Zurich(Switzerland): ...



from _typeshed import Incomplete
from argparse import Namespace
from ast import AST
from collections.abc import Generator
from logging import Logger
from tokenize import TokenInfo
from typing import Any, Final
from typing_extensions import TypeAlias

from .plugins.finder import LoadedPlugin

LOG: Logger
NEWLINE: Final[frozenset[int]]
SKIP_TOKENS: Final[frozenset[int]]

_LogicalMapping: TypeAlias = list[tuple[int, tuple[int, int]]]
_Logical: TypeAlias = tuple[list[str], list[str], _LogicalMapping]

class FileProcessor:
    noqa: bool
    options: Incomplete
    filename: Incomplete
    lines: Incomplete
    blank_before: int
    blank_lines: int
    checker_state: Incomplete
    hang_closing: Incomplete
    indent_char: Incomplete
    indent_level: int
    indent_size: Incomplete
    line_number: int
    logical_line: str
    max_line_length: Incomplete
    max_doc_length: Incomplete
    multiline: bool
    previous_indent_level: int
    previous_logical: str
    previous_unindented_logical_line: str
    tokens: Incomplete
    total_lines: Incomplete
    verbose: Incomplete
    statistics: Incomplete
    def __init__(self, filename: str, options: Namespace, lines: list[str] | None = None) -> None: ...
    @property
    def file_tokens(self) -> list[TokenInfo]: ...
    def tstring_start(self, lineno: int) -> None: ...
    def fstring_start(self, lineno: int) -> None: ...
    def multiline_string(self, token: TokenInfo) -> Generator[str, None, None]: ...
    def reset_blank_before(self) -> None: ...
    def delete_first_token(self) -> None: ...
    def visited_new_blank_line(self) -> None: ...
    def update_state(self, mapping: _LogicalMapping) -> None: ...
    def update_checker_state_for(self, plugin: LoadedPlugin) -> None: ...
    def next_logical_line(self) -> None: ...
    def build_logical_line_tokens(self) -> _Logical: ...
    def build_ast(self) -> AST: ...
    def build_logical_line(self) -> tuple[str, str, _LogicalMapping]: ...
    def keyword_arguments_for(self, parameters: dict[str, bool], arguments: dict[str, Any]) -> dict[str, Any]: ...
    def generate_tokens(self) -> Generator[TokenInfo, None, None]: ...
    def noqa_line_for(self, line_number: int) -> str | None: ...
    def next_line(self) -> str: ...
    def read_lines(self) -> list[str]: ...
    def read_lines_from_filename(self) -> list[str]: ...
    def read_lines_from_stdin(self) -> list[str]: ...
    def should_ignore_file(self) -> bool: ...
    def strip_utf_bom(self) -> None: ...

def is_eol_token(token: TokenInfo) -> bool: ...
def is_multiline_string(token: TokenInfo) -> bool: ...
def token_is_newline(token: TokenInfo) -> bool: ...
def count_parentheses(current_parentheses_count: int, token_text: str) -> int: ...
def expand_indent(line: str) -> int: ...
def mutate_string(text: str) -> str: ...



############################################################################
# Original work Copyright 2017 Palantir Technologies, Inc.                 #
# Original work licensed under the MIT License.                            #
# See ThirdPartyNotices.txt in the project root for license information.   #
# All modifications Copyright (c) Open Law Library. All rights reserved.   #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License")           #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#     http: // www.apache.org/licenses/LICENSE-2.0                         #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
############################################################################
import io
import logging
import os
import re
from typing import List, Optional, Pattern

from lsprotocol import types

from pygls.uris import to_fs_path
from .position_codec import PositionCodec

# TODO: this is not the best e.g. we capture numbers
RE_END_WORD = re.compile("^[A-Za-z_0-9]*")
RE_START_WORD = re.compile("[A-Za-z_0-9]*$")

logger = logging.getLogger(__name__)


class TextDocument(object):
    def __init__(
        self,
        uri: str,
        source: Optional[str] = None,
        version: Optional[int] = None,
        language_id: Optional[str] = None,
        local: bool = True,
        sync_kind: types.TextDocumentSyncKind = types.TextDocumentSyncKind.Incremental,
        position_codec: Optional[PositionCodec] = None,
    ):
        self.uri = uri
        self.version = version
        path = to_fs_path(uri)
        if path is None:
            raise Exception("`path` cannot be None")
        self.path = path
        self.language_id = language_id
        self.filename: Optional[str] = os.path.basename(self.path)

        self._local = local
        self._source = source

        self._is_sync_kind_full = sync_kind == types.TextDocumentSyncKind.Full
        self._is_sync_kind_incremental = (
            sync_kind == types.TextDocumentSyncKind.Incremental
        )
        self._is_sync_kind_none = sync_kind == types.TextDocumentSyncKind.None_

        self._position_codec = position_codec if position_codec else PositionCodec()

    def __str__(self):
        return str(self.uri)

    @property
    def position_codec(self) -> PositionCodec:
        return self._position_codec

    def _apply_incremental_change(
        self, change: types.TextDocumentContentChangeEvent_Type1
    ) -> None:
        """Apply an ``Incremental`` text change to the document"""
        lines = self.lines
        text = change.text
        change_range = change.range

        range = self._position_codec.range_from_client_units(lines, change_range)
        start_line = range.start.line
        start_col = range.start.character
        end_line = range.end.line
        end_col = range.end.character

        # Check for an edit occurring at the very end of the file
        if start_line == len(lines):
            self._source = self.source + text
            return

        new = io.StringIO()

        # Iterate over the existing document until we hit the edit range,
        # at which point we write the new text, then loop until we hit
        # the end of the range and continue writing.
        for i, line in enumerate(lines):
            if i < start_line:
                new.write(line)
                continue

            if i > end_line:
                new.write(line)
                continue

            if i == start_line:
                new.write(line[:start_col])
                new.write(text)

            if i == end_line:
                new.write(line[end_col:])

        self._source = new.getvalue()

    def _apply_full_change(self, change: types.TextDocumentContentChangeEvent) -> None:
        """Apply a ``Full`` text change to the document."""
        self._source = change.text

    def _apply_none_change(self, _: types.TextDocumentContentChangeEvent) -> None:
        """Apply a ``None`` text change to the document

        Currently does nothing, provided for consistency.
        """
        pass

    def apply_change(self, change: types.TextDocumentContentChangeEvent) -> None:
        """Apply a text change to a document, considering TextDocumentSyncKind

        Performs either
        :attr:`~lsprotocol.types.TextDocumentSyncKind.Incremental`,
        :attr:`~lsprotocol.types.TextDocumentSyncKind.Full`, or no synchronization
        based on both the client request and server capabilities.

        .. admonition:: ``Incremental`` versus ``Full`` synchronization

           Even if a server accepts ``Incremantal`` SyncKinds, clients may request
           a ``Full`` SyncKind. In LSP 3.x, clients make this request by omitting
           both Range and RangeLength from their request. Consequently, the
           attributes "range" and "rangeLength" will be missing from ``Full``
           content update client requests in the pygls Python library.

        """
        if isinstance(change, types.TextDocumentContentChangeEvent_Type1):
            if self._is_sync_kind_incremental:
                self._apply_incremental_change(change)
                return
            # Log an error, but still perform full update to preserve existing
            # assumptions in test_document/test_document_full_edit. Test breaks
            # otherwise, and fixing the tests would require a broader fix to
            # protocol.py.
            logger.error(
                "Unsupported client-provided TextDocumentContentChangeEvent. "
                "Please update / submit a Pull Request to your LSP client."
            )

        if self._is_sync_kind_none:
            self._apply_none_change(change)
        else:
            self._apply_full_change(change)

    @property
    def lines(self) -> List[str]:
        return self.source.splitlines(True)

    def offset_at_position(self, client_position: types.Position) -> int:
        """Return the character offset pointed at by the given client_position."""
        lines = self.lines
        server_position = self._position_codec.position_from_client_units(
            lines, client_position
        )
        row, col = server_position.line, server_position.character
        return col + sum(
            self._position_codec.client_num_units(line) for line in lines[:row]
        )

    @property
    def source(self) -> str:
        if self._source is None:
            with io.open(self.path, "r", encoding="utf-8") as f:
                return f.read()
        return self._source

    def word_at_position(
        self,
        client_position: types.Position,
        re_start_word: Pattern[str] = RE_START_WORD,
        re_end_word: Pattern[str] = RE_END_WORD,
    ) -> str:
        """Return the word at position.

        The word is constructed in two halves, the first half is found by taking
        the first match of ``re_start_word`` on the line up until
        ``position.character``.

        The second half is found by taking ``position.character`` up until the
        last match of ``re_end_word`` on the line.

        :func:`python:re.findall` is used to find the matches.

        Parameters
        ----------
        position
           The line and character offset.

        re_start_word
           The regular expression for extracting the word backward from
           position. The default pattern is ``[A-Za-z_0-9]*$``.

        re_end_word
           The regular expression for extracting the word forward from
           position. The default pattern is ``^[A-Za-z_0-9]*``.

        Returns
        -------
        str
           The word (obtained by concatenating the two matches) at position.
        """
        lines = self.lines
        if client_position.line >= len(lines):
            return ""

        server_position = self._position_codec.position_from_client_units(
            lines, client_position
        )
        row, col = server_position.line, server_position.character
        line = lines[row]
        # Split word in two
        start = line[:col]
        end = line[col:]

        # Take end of start and start of end to find word
        # These are guaranteed to match, even if they match the empty string
        m_start = re_start_word.findall(start)
        m_end = re_end_word.findall(end)

        return m_start[0] + m_end[-1]



from collections.abc import Callable, Iterable

from networkx.algorithms.flow import edmonds_karp
from networkx.classes.digraph import DiGraph
from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = [
    "average_node_connectivity",
    "local_node_connectivity",
    "node_connectivity",
    "local_edge_connectivity",
    "edge_connectivity",
    "all_pairs_node_connectivity",
]
default_flow_func = edmonds_karp

@_dispatchable
def local_node_connectivity(
    G: Graph[_Node],
    s: _Node,
    t: _Node,
    flow_func: Callable[[DiGraph[_Node], _Node, _Node], DiGraph[_Node]] | None = None,
    auxiliary: DiGraph[_Node] | None = None,
    residual: DiGraph[_Node] | None = None,
    cutoff: float | None = None,
) -> float: ...
@_dispatchable
def node_connectivity(
    G: Graph[_Node],
    s: _Node | None = None,
    t: _Node | None = None,
    flow_func: Callable[[DiGraph[_Node], _Node, _Node], DiGraph[_Node]] | None = None,
) -> float: ...
@_dispatchable
def average_node_connectivity(
    G: Graph[_Node], flow_func: Callable[[DiGraph[_Node], _Node, _Node], DiGraph[_Node]] | None = None
) -> float: ...
@_dispatchable
def all_pairs_node_connectivity(
    G: Graph[_Node],
    nbunch: Iterable[tuple[_Node, _Node]] | None = None,
    flow_func: Callable[[DiGraph[_Node], _Node, _Node], DiGraph[_Node]] | None = None,
) -> dict[_Node, dict[_Node, float]]: ...
@_dispatchable
def local_edge_connectivity(
    G: Graph[_Node],
    s: _Node,
    t: _Node,
    flow_func: Callable[[DiGraph[_Node], _Node, _Node], DiGraph[_Node]] | None = None,
    auxiliary: DiGraph[_Node] | None = None,
    residual: DiGraph[_Node] | None = None,
    cutoff: float | None = None,
) -> float: ...
@_dispatchable
def edge_connectivity(
    G: Graph[_Node],
    s: _Node | None = None,
    t: _Node | None = None,
    flow_func: Callable[[DiGraph[_Node], _Node, _Node], DiGraph[_Node]] | None = None,
    cutoff: float | None = None,
) -> float: ...



from _typeshed import Incomplete

from authlib.oauth2.rfc7009 import RevocationEndpoint

class JWTRevocationEndpoint(RevocationEndpoint):
    issuer: Incomplete
    def __init__(self, issuer, server=None, *args, **kwargs) -> None: ...
    def authenticate_token(self, request, client) -> None: ...
    def get_jwks(self) -> None: ...



import sys
from typing import Final

if sys.platform == "win32":
    from . import win32

    class WinColor:
        BLACK: Final = 0
        BLUE: Final = 1
        GREEN: Final = 2
        CYAN: Final = 3
        RED: Final = 4
        MAGENTA: Final = 5
        YELLOW: Final = 6
        GREY: Final = 7

    class WinStyle:
        NORMAL: Final = 0x00
        BRIGHT: Final = 0x08
        BRIGHT_BACKGROUND: Final = 0x80

    class WinTerm:
        def __init__(self) -> None: ...
        def get_attrs(self) -> int: ...
        def set_attrs(self, value: int) -> None: ...
        def reset_all(self, on_stderr: bool | None = None) -> None: ...
        def fore(self, fore: int | None = None, light: bool = False, on_stderr: bool = False) -> None: ...
        def back(self, back: int | None = None, light: bool = False, on_stderr: bool = False) -> None: ...
        def style(self, style: int | None = None, on_stderr: bool = False) -> None: ...
        def set_console(self, attrs: int | None = None, on_stderr: bool = False) -> None: ...
        def get_position(self, handle: int) -> win32.COORD: ...
        def set_cursor_position(self, position: win32.COORD | None = None, on_stderr: bool = False) -> None: ...
        def cursor_adjust(self, x: int, y: int, on_stderr: bool = False) -> None: ...
        def erase_screen(self, mode: int = 0, on_stderr: bool = False) -> None: ...
        def erase_line(self, mode: int = 0, on_stderr: bool = False) -> None: ...
        def set_title(self, title: str) -> None: ...

def enable_vt_processing(fd: int) -> bool: ...



from _typeshed import Incomplete

from braintree.error_result import ErrorResult
from braintree.graphql import CreateCustomerSessionInput, CustomerRecommendationsInput, UpdateCustomerSessionInput
from braintree.successful_result import SuccessfulResult

class CustomerSessionGateway:
    gateway: Incomplete
    graphql_client: Incomplete
    def __init__(self, gateway) -> None: ...
    def create_customer_session(self, customer_session_input: CreateCustomerSessionInput) -> SuccessfulResult | ErrorResult: ...
    def update_customer_session(
        self, update_customer_session_input: UpdateCustomerSessionInput
    ) -> SuccessfulResult | ErrorResult: ...
    def get_customer_recommendations(
        self, get_customer_recommendations_input: CustomerRecommendationsInput
    ) -> SuccessfulResult | ErrorResult: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class DebugTensorWatch(google.protobuf.message.Message):
    """Option for watching a node in TensorFlow Debugger (tfdbg)."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NODE_NAME_FIELD_NUMBER: builtins.int
    OUTPUT_SLOT_FIELD_NUMBER: builtins.int
    DEBUG_OPS_FIELD_NUMBER: builtins.int
    DEBUG_URLS_FIELD_NUMBER: builtins.int
    TOLERATE_DEBUG_OP_CREATION_FAILURES_FIELD_NUMBER: builtins.int
    node_name: builtins.str
    """Name of the node to watch.
    Use "*" for wildcard. But note: currently, regex is not supported in
    general.
    """
    output_slot: builtins.int
    """Output slot to watch.
    The semantics of output_slot == -1 is that all outputs of the node
    will be watched (i.e., a wildcard).
    Other negative values of output_slot are invalid and will lead to
    errors currently.
    """
    tolerate_debug_op_creation_failures: builtins.bool
    """Do not error out if debug op creation fails (e.g., due to dtype
    incompatibility). Instead, just log the failure.
    """
    @property
    def debug_ops(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """Name(s) of the debugging op(s).
        One or more than one probes on a tensor.
        e.g., {"DebugIdentity", "DebugNanCount"}
        """

    @property
    def debug_urls(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """URL(s) for debug targets(s).

        Supported URL formats are:
          - file:///foo/tfdbg_dump: Writes out Event content to file
            /foo/tfdbg_dump.  Assumes all directories can be created if they don't
            already exist.
          - grpc://localhost:11011: Sends an RPC request to an EventListener
            service running at localhost:11011 with the event.
          - memcbk:///event_key: Routes tensors to clients using the
            callback registered with the DebugCallbackRegistry for event_key.

        Each debug op listed in debug_ops will publish its output tensor (debug
        signal) to all URLs in debug_urls.

        N.B. Session::Run() supports concurrent invocations of the same inputs
        (feed keys), outputs and target nodes. If such concurrent invocations
        are to be debugged, the callers of Session::Run() must use distinct
        debug_urls to make sure that the streamed or dumped events do not overlap
        among the invocations.
        TODO(cais): More visible documentation of this in g3docs.
        """

    def __init__(
        self,
        *,
        node_name: builtins.str | None = ...,
        output_slot: builtins.int | None = ...,
        debug_ops: collections.abc.Iterable[builtins.str] | None = ...,
        debug_urls: collections.abc.Iterable[builtins.str] | None = ...,
        tolerate_debug_op_creation_failures: builtins.bool | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "debug_ops",
            b"debug_ops",
            "debug_urls",
            b"debug_urls",
            "node_name",
            b"node_name",
            "output_slot",
            b"output_slot",
            "tolerate_debug_op_creation_failures",
            b"tolerate_debug_op_creation_failures",
        ],
    ) -> None: ...

global___DebugTensorWatch = DebugTensorWatch

@typing.final
class DebugOptions(google.protobuf.message.Message):
    """Options for initializing DebuggerState in TensorFlow Debugger (tfdbg)."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DEBUG_TENSOR_WATCH_OPTS_FIELD_NUMBER: builtins.int
    GLOBAL_STEP_FIELD_NUMBER: builtins.int
    RESET_DISK_BYTE_USAGE_FIELD_NUMBER: builtins.int
    global_step: builtins.int
    """Caller-specified global step count.
    Note that this is distinct from the session run count and the executor
    step count.
    """
    reset_disk_byte_usage: builtins.bool
    """Whether the total disk usage of tfdbg is to be reset to zero
    in this Session.run call. This is used by wrappers and hooks
    such as the local CLI ones to indicate that the dumped tensors
    are cleaned up from the disk after each Session.run.
    """
    @property
    def debug_tensor_watch_opts(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___DebugTensorWatch]:
        """Debugging options"""

    def __init__(
        self,
        *,
        debug_tensor_watch_opts: collections.abc.Iterable[global___DebugTensorWatch] | None = ...,
        global_step: builtins.int | None = ...,
        reset_disk_byte_usage: builtins.bool | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "debug_tensor_watch_opts",
            b"debug_tensor_watch_opts",
            "global_step",
            b"global_step",
            "reset_disk_byte_usage",
            b"reset_disk_byte_usage",
        ],
    ) -> None: ...

global___DebugOptions = DebugOptions

@typing.final
class DebuggedSourceFile(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    HOST_FIELD_NUMBER: builtins.int
    FILE_PATH_FIELD_NUMBER: builtins.int
    LAST_MODIFIED_FIELD_NUMBER: builtins.int
    BYTES_FIELD_NUMBER: builtins.int
    LINES_FIELD_NUMBER: builtins.int
    host: builtins.str
    """The host name on which a source code file is located."""
    file_path: builtins.str
    """Path to the source code file."""
    last_modified: builtins.int
    """The timestamp at which the source code file is last modified."""
    bytes: builtins.int
    """Byte size of the file."""
    @property
    def lines(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """Line-by-line content of the source code file."""

    def __init__(
        self,
        *,
        host: builtins.str | None = ...,
        file_path: builtins.str | None = ...,
        last_modified: builtins.int | None = ...,
        bytes: builtins.int | None = ...,
        lines: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "bytes", b"bytes", "file_path", b"file_path", "host", b"host", "last_modified", b"last_modified", "lines", b"lines"
        ],
    ) -> None: ...

global___DebuggedSourceFile = DebuggedSourceFile

@typing.final
class DebuggedSourceFiles(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SOURCE_FILES_FIELD_NUMBER: builtins.int
    @property
    def source_files(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___DebuggedSourceFile]:
        """A collection of source code files."""

    def __init__(self, *, source_files: collections.abc.Iterable[global___DebuggedSourceFile] | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["source_files", b"source_files"]) -> None: ...

global___DebuggedSourceFiles = DebuggedSourceFiles



from _typeshed import Incomplete
from typing import ClassVar
from typing_extensions import deprecated

import passlib.utils.handlers as uh

class HexDigestHash(uh.StaticHandler):
    checksum_chars: ClassVar[str]
    supported: ClassVar[bool]

def create_hex_hash(digest, module="passlib.handlers.digests", django_name=None, required: bool = True): ...

hex_md4: Incomplete
hex_md5: Incomplete
hex_sha1: Incomplete
hex_sha256: Incomplete
hex_sha512: Incomplete

class htdigest(uh.MinimalHandler):
    name: ClassVar[str]
    default_encoding: ClassVar[str]
    setting_kwds: ClassVar[tuple[str, ...]]
    context_kwds: ClassVar[tuple[str, ...]]
    @classmethod
    def hash(cls, secret, user, realm, encoding=None): ...  # type: ignore[override]
    @classmethod
    def verify(cls, secret, hash, user, realm, encoding: str = "utf-8"): ...  # type: ignore[override]
    @classmethod
    def identify(cls, hash): ...
    @deprecated("Deprecated since Passlib 1.7, will be removed in 2.0")
    @classmethod
    def genconfig(cls): ...  # type: ignore[override]
    @deprecated("Deprecated since Passlib 1.7, will be removed in 2.0")
    @classmethod
    def genhash(cls, secret, config, user, realm, encoding=None): ...  # type: ignore[override]

__all__ = ["create_hex_hash", "hex_md4", "hex_md5", "hex_sha1", "hex_sha256", "hex_sha512"]



from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["tutte_polynomial", "chromatic_polynomial"]

@_dispatchable
def tutte_polynomial(G: Graph[_Node]): ...
@_dispatchable
def chromatic_polynomial(G: Graph[_Node]): ...



from typing import Any

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["cytoscape_data", "cytoscape_graph"]

# Any: Complex type union
def cytoscape_data(G: Graph[_Node], name: str = "name", ident: str = "id") -> dict[str, Any]: ...
@_dispatchable
def cytoscape_graph(data, name: str = "name", ident: str = "id"): ...



from asyncio.events import AbstractEventLoop
from collections.abc import Awaitable, Callable, Generator
from concurrent.futures import Executor
from contextlib import AbstractAsyncContextManager
from types import TracebackType
from typing import Any, BinaryIO, Generic, TextIO, TypeVar
from typing_extensions import Self

_T = TypeVar("_T")
_V_co = TypeVar("_V_co", covariant=True)

def wrap(func: Callable[..., _T]) -> Callable[..., Awaitable[_T]]: ...

class AsyncBase(Generic[_T]):
    def __init__(self, file: TextIO | BinaryIO | None, loop: AbstractEventLoop | None, executor: Executor | None) -> None: ...
    def __aiter__(self) -> Self: ...
    async def __anext__(self) -> _T: ...

class AsyncIndirectBase(AsyncBase[_T]):
    def __init__(
        self, name: str, loop: AbstractEventLoop | None, executor: Executor | None, indirect: Callable[[], TextIO | BinaryIO]
    ) -> None: ...

class AiofilesContextManager(Awaitable[_V_co], AbstractAsyncContextManager[_V_co]):
    __slots__ = ("_coro", "_obj")
    def __init__(self, coro: Awaitable[_V_co]) -> None: ...
    def __await__(self) -> Generator[Any, Any, _V_co]: ...
    async def __aenter__(self) -> _V_co: ...
    async def __aexit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...



from _typeshed import Incomplete, SupportsGetItem

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["core_number", "k_core", "k_shell", "k_crust", "k_corona", "k_truss", "onion_layers"]

@_dispatchable
def core_number(G: Graph[_Node]) -> dict[Incomplete, Incomplete]: ...
@_dispatchable
def k_core(G: Graph[_Node], k: int | None = None, core_number: SupportsGetItem[Incomplete, Incomplete] | None = None): ...
@_dispatchable
def k_shell(G: Graph[_Node], k: int | None = None, core_number: SupportsGetItem[Incomplete, Incomplete] | None = None): ...
@_dispatchable
def k_crust(G: Graph[_Node], k: int | None = None, core_number: SupportsGetItem[Incomplete, Incomplete] | None = None): ...
@_dispatchable
def k_corona(G: Graph[_Node], k: int, core_number: SupportsGetItem[Incomplete, Incomplete] | None = None): ...
@_dispatchable
def k_truss(G: Graph[_Node], k: int): ...
@_dispatchable
def onion_layers(G: Graph[_Node]) -> dict[Incomplete, Incomplete]: ...



from typing import Final

__version__: Final[str]

class PDFPathObject:
    def __init__(self, code=None) -> None: ...
    def getCode(self): ...
    def moveTo(self, x, y) -> None: ...
    def lineTo(self, x, y) -> None: ...
    def curveTo(self, x1, y1, x2, y2, x3, y3) -> None: ...
    def arc(self, x1, y1, x2, y2, startAng: int = 0, extent: int = 90) -> None: ...
    def arcTo(self, x1, y1, x2, y2, startAng: int = 0, extent: int = 90) -> None: ...
    def rect(self, x, y, width, height) -> None: ...
    def ellipse(self, x, y, width, height) -> None: ...
    def circle(self, x_cen, y_cen, r) -> None: ...
    def roundRect(self, x, y, width, height, radius) -> None: ...
    def close(self) -> None: ...



from __future__ import annotations

from wtforms import SelectField

# any way we can specify the choices inline with a literal should work

# tuple of tuples
SelectField(choices=(("", ""),))
SelectField(choices=((1, "1"),))
SelectField(choices=(("", "", {}),))
SelectField(choices=((True, "t", {}),))
SelectField(choices=((True, "t"), (False, "f", {})))

# list of tuples
SelectField(choices=[("", "")])
SelectField(choices=[(1, "1")])
SelectField(choices=[("", "", {})])
SelectField(choices=[(True, "t", {})])
SelectField(choices=[(True, "t"), (False, "f", {})])

# dict of tuple of tuples
SelectField(choices={"a": (("", ""),)})
SelectField(choices={"a": ((1, "1"),)})
SelectField(choices={"a": (("", "", {}),)})
SelectField(choices={"a": ((True, "t", {}),)})
SelectField(choices={"a": ((True, "t"), (False, "f", {}))})
SelectField(choices={"a": ((True, "", {}),), "b": ((False, "f"),)})

# dict of list of tuples
SelectField(choices={"a": [("", "")]})
SelectField(choices={"a": [(1, "1")]})
SelectField(choices={"a": [("", "", {})]})
SelectField(choices={"a": [(True, "t", {})]})
SelectField(choices={"a": [(True, "t"), (False, "f", {})]})
SelectField(choices={"a": [(True, "", {})], "b": [(False, "f")]})

# the same should be true for lambdas

# tuple of tuples
SelectField(choices=lambda: (("", ""),))
SelectField(choices=lambda: ((1, "1"),))
SelectField(choices=lambda: (("", "", {}),))
SelectField(choices=lambda: ((True, "t", {}),))
SelectField(choices=lambda: ((True, "t"), (False, "f", {})))

# list of tuples
SelectField(choices=lambda: [("", "")])
SelectField(choices=lambda: [(1, "1")])
SelectField(choices=lambda: [("", "", {})])
SelectField(choices=lambda: [(True, "t", {})])
SelectField(choices=lambda: [(True, "t"), (False, "f", {})])

# dict of tuple of tuples
SelectField(choices=lambda: {"a": (("", ""),)})
SelectField(choices=lambda: {"a": ((1, "1"),)})
SelectField(choices=lambda: {"a": (("", "", {}),)})
SelectField(choices=lambda: {"a": ((True, "t", {}),)})
SelectField(choices=lambda: {"a": ((True, "t"), (False, "f", {}))})
SelectField(choices=lambda: {"a": ((True, "", {}),), "b": ((False, "f"),)})

# dict of list of tuples
SelectField(choices=lambda: {"a": [("", "")]})
SelectField(choices=lambda: {"a": [(1, "1")]})
SelectField(choices=lambda: {"a": [("", "", {})]})
SelectField(choices=lambda: {"a": [(True, "t", {})]})
SelectField(choices=lambda: {"a": [(True, "t"), (False, "f", {})]})
SelectField(choices=lambda: {"a": [(True, "", {})], "b": [(False, "f")]})



from _typeshed import Incomplete
from collections.abc import Callable, Mapping
from re import Pattern
from typing import Protocol, TypeVar, type_check_only
from typing_extensions import ParamSpec

CUR_OS: str
IS_WIN: bool
IS_NIX: bool
RE_ANSI: Pattern[str]

class FormatReplace:
    replace: str
    format_called: int
    def __init__(self, replace: str = "") -> None: ...
    def __format__(self, _) -> str: ...

@type_check_only
class _Has__Comparable(Protocol):
    _comparable: Incomplete

class Comparable:
    _comparable: Incomplete
    def __lt__(self, other: _Has__Comparable) -> bool: ...
    def __le__(self, other: _Has__Comparable) -> bool: ...
    def __eq__(self, other: _Has__Comparable) -> bool: ...  # type: ignore[override]
    def __ne__(self, other: _Has__Comparable) -> bool: ...  # type: ignore[override]
    def __gt__(self, other: _Has__Comparable) -> bool: ...
    def __ge__(self, other: _Has__Comparable) -> bool: ...

class ObjectWrapper:
    def __getattr__(self, name: str): ...
    def __setattr__(self, name: str, value) -> None: ...
    def wrapper_getattr(self, name): ...
    def wrapper_setattr(self, name, value): ...
    def __init__(self, wrapped) -> None: ...

class SimpleTextIOWrapper(ObjectWrapper):
    def __init__(self, wrapped, encoding) -> None: ...
    def write(self, s: str): ...
    def __eq__(self, other: object) -> bool: ...

_P = ParamSpec("_P")
_R = TypeVar("_R")

class DisableOnWriteError(ObjectWrapper):
    @staticmethod
    def disable_on_exception(tqdm_instance, func: Callable[_P, _R]) -> Callable[_P, _R]: ...
    def __init__(self, wrapped, tqdm_instance) -> None: ...
    def __eq__(self, other: object) -> bool: ...

class CallbackIOWrapper(ObjectWrapper):
    def __init__(self, callback: Callable[[int], object], stream, method: str = "read") -> None: ...

def disp_len(data: str) -> int: ...
def disp_trim(data: str, length: int) -> str: ...
def envwrap(
    prefix: str, types: Mapping[str, Callable[[Incomplete], Incomplete]] | None = None, is_method: bool = False
) -> Callable[[Callable[_P, _R]], Callable[_P, _R]]: ...



from _typeshed import Incomplete, ReadableBuffer, StrPath, Unused
from collections.abc import Generator, Iterable
from typing import Protocol, type_check_only
from typing_extensions import TypeAlias

from openpyxl.cell import _CellOrMergedCell
from openpyxl.worksheet._write_only import WriteOnlyWorksheet
from openpyxl.worksheet.worksheet import Worksheet

# WorksheetWriter.read has an explicit BytesIO branch. Let's make sure this protocol is viable for BytesIO too.
@type_check_only
class _SupportsCloseAndWrite(Protocol):
    def write(self, buffer: ReadableBuffer, /) -> Unused: ...
    def close(self) -> Unused: ...

# et_xmlfile.xmlfile accepts a str | _SupportsCloseAndWrite
# lxml.etree.xmlfile should accept a StrPath | _SupportsClose https://lxml.de/api/lxml.etree.xmlfile-class.html
_OutType: TypeAlias = _SupportsCloseAndWrite | StrPath

ALL_TEMP_FILES: list[str]

def create_temporary_file(suffix: str = "") -> str: ...

class WorksheetWriter:
    ws: Worksheet | WriteOnlyWorksheet
    out: _OutType
    xf: Generator[Incomplete | None, Incomplete, None]
    def __init__(self, ws: Worksheet | WriteOnlyWorksheet, out: _OutType | None = None) -> None: ...
    def write_properties(self) -> None: ...
    def write_dimensions(self) -> None: ...
    def write_format(self) -> None: ...
    def write_views(self) -> None: ...
    def write_cols(self) -> None: ...
    def write_top(self) -> None: ...
    def rows(self) -> list[tuple[int, list[_CellOrMergedCell]]]: ...
    def write_rows(self) -> None: ...
    def write_row(self, xf, row: Iterable[_CellOrMergedCell], row_idx) -> None: ...
    def write_protection(self) -> None: ...
    def write_scenarios(self) -> None: ...
    def write_filter(self) -> None: ...
    def write_sort(self) -> None: ...
    def write_merged_cells(self) -> None: ...
    def write_formatting(self) -> None: ...
    def write_validations(self) -> None: ...
    def write_hyperlinks(self) -> None: ...
    def write_print(self) -> None: ...
    def write_margins(self) -> None: ...
    def write_page(self) -> None: ...
    def write_header(self) -> None: ...
    def write_breaks(self) -> None: ...
    def write_drawings(self) -> None: ...
    def write_legacy(self) -> None: ...
    def write_tables(self) -> None: ...
    def get_stream(self) -> Generator[Incomplete | None, bool | None, None]: ...
    def write_tail(self) -> None: ...
    def write(self) -> None: ...
    def close(self) -> None: ...
    def read(self) -> bytes: ...
    def cleanup(self) -> None: ...



from typing import Any

class Error(Exception): ...

class ContextError(Error):
    def __init__(self) -> None: ...

class BadValueError(Error): ...
class BadArgumentError(Error): ...
class BadRequestError(Error): ...
class Rollback(Error): ...
class BadQueryError(Error): ...

class BadFilterError(Error):
    filter: Any
    def __init__(self, filter) -> None: ...

class NoLongerImplementedError(NotImplementedError):
    def __init__(self) -> None: ...

class Cancelled(Error): ...
class NestedRetryException(Error): ...



from typing import Any

import pandas as pd

class GeoSeriesAccessor:
    def __init__(self, series: pd.Series[Any]) -> None: ...  # Cannot use pd.Series[BaseGeometry]
    def __getattr__(self, name: str) -> Any: ...  # Delegate all attributes to the GeoSeries



from _typeshed import Incomplete
from typing import Any

class Comment:
    content: Incomplete
    author: Incomplete
    height: Incomplete
    width: Incomplete
    def __init__(self, text, author, height: int = 79, width: int = 144) -> None: ...
    @property
    def parent(self) -> Any: ...  # AnyOf[Cell, MergedCell, ReadOnlyCell]
    def __eq__(self, other: Comment) -> bool: ...  # type: ignore[override]
    def __copy__(self): ...
    def bind(self, cell) -> None: ...
    def unbind(self) -> None: ...
    @property
    def text(self) -> str: ...
    @text.setter
    def text(self, value: str) -> None: ...



from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["build_auxiliary_node_connectivity", "build_auxiliary_edge_connectivity"]

@_dispatchable
def build_auxiliary_node_connectivity(G: Graph[_Node]): ...
@_dispatchable
def build_auxiliary_edge_connectivity(G: Graph[_Node]): ...



from typing import ClassVar

from authlib.jose.rfc7517 import AsymmetricKey
from cryptography.hazmat.primitives.asymmetric.rsa import RSAPrivateKey, RSAPublicKey

class RSAKey(AsymmetricKey):
    kty: str
    PUBLIC_KEY_CLS: ClassVar[type]
    PRIVATE_KEY_CLS: ClassVar[type]
    PUBLIC_KEY_FIELDS: ClassVar[list[str]]
    PRIVATE_KEY_FIELDS: ClassVar[list[str]]
    REQUIRED_JSON_FIELDS: ClassVar[list[str]]
    SSH_PUBLIC_PREFIX: ClassVar[bytes]
    def dumps_private_key(self) -> dict[str, str]: ...
    def dumps_public_key(self) -> dict[str, str]: ...
    def load_private_key(self) -> RSAPrivateKey: ...
    def load_public_key(self) -> RSAPublicKey: ...
    @classmethod
    def generate_key(cls, key_size: int = 2048, options=None, is_private: bool = False) -> RSAKey: ...
    @classmethod
    def import_dict_key(cls, raw, options=None): ...

def has_all_prime_factors(obj) -> bool: ...



from hvac.api.vault_api_base import VaultApiBase

DEFAULT_MOUNT_POINT: str

class Pki(VaultApiBase):
    def read_ca_certificate(self, mount_point="pki"): ...
    def read_ca_certificate_chain(self, mount_point="pki"): ...
    def read_certificate(self, serial, mount_point="pki"): ...
    def list_certificates(self, mount_point="pki"): ...
    def submit_ca_information(self, pem_bundle, mount_point="pki"): ...
    def read_crl_configuration(self, mount_point="pki"): ...
    def set_crl_configuration(self, expiry=None, disable=None, extra_params=None, mount_point="pki"): ...
    def read_urls(self, mount_point="pki"): ...
    def set_urls(self, params, mount_point="pki"): ...
    def read_crl(self, mount_point="pki"): ...
    def rotate_crl(self, mount_point="pki"): ...
    def generate_intermediate(self, type, common_name, extra_params=None, mount_point="pki", wrap_ttl=None): ...
    def set_signed_intermediate(self, certificate, mount_point="pki"): ...
    def generate_certificate(self, name, common_name, extra_params=None, mount_point="pki", wrap_ttl=None): ...
    def revoke_certificate(self, serial_number, mount_point="pki"): ...
    def create_or_update_role(self, name, extra_params=None, mount_point="pki"): ...
    def read_role(self, name, mount_point="pki"): ...
    def list_roles(self, mount_point="pki"): ...
    def delete_role(self, name, mount_point="pki"): ...
    def generate_root(self, type, common_name, extra_params=None, mount_point="pki", wrap_ttl=None): ...
    def delete_root(self, mount_point="pki"): ...
    def sign_intermediate(self, csr, common_name, extra_params=None, mount_point="pki"): ...
    def sign_self_issued(self, certificate, mount_point="pki"): ...
    def sign_certificate(self, name, csr, common_name, extra_params=None, mount_point="pki"): ...
    def sign_verbatim(self, csr, name: bool = False, extra_params=None, mount_point="pki"): ...
    def tidy(self, extra_params=None, mount_point="pki"): ...
    def read_issuer(self, issuer_ref, mount_point="pki"): ...
    def list_issuers(self, mount_point="pki"): ...
    def update_issuer(self, issuer_ref, extra_params=None, mount_point="pki"): ...
    def revoke_issuer(self, issuer_ref, mount_point="pki"): ...
    def delete_issuer(self, issuer_ref, mount_point="pki"): ...



from collections.abc import Callable, Iterable
from datetime import timedelta
from logging import Logger
from typing import Any

import flask

LOG: Logger

class CORS:
    def __init__(
        self,
        app: flask.Flask | flask.Blueprint | None = None,
        *,
        resources: dict[str, dict[str, Any]] | list[str] | str | None = ...,
        origins: str | list[str] | None = ...,
        methods: str | list[str] | None = ...,
        expose_headers: str | list[str] | None = ...,
        allow_headers: str | list[str] | None = ...,
        supports_credentials: bool | None = ...,
        max_age: timedelta | int | str | None = ...,
        send_wildcard: bool | None = ...,
        vary_header: bool | None = ...,
        **kwargs: Any,
    ) -> None: ...
    def init_app(
        self,
        app: flask.Flask,
        *,
        resources: dict[str, dict[str, Any]] | list[str] | str = ...,
        origins: str | list[str] = ...,
        methods: str | list[str] = ...,
        expose_headers: str | list[str] = ...,
        allow_headers: str | list[str] = ...,
        supports_credentials: bool = ...,
        max_age: timedelta | int | str | None = ...,
        send_wildcard: bool = ...,
        vary_header: bool = ...,
        **kwargs: Any,
    ) -> None: ...

def make_after_request_function(resources: Iterable[tuple[str, dict[str, Any]]]) -> Callable[..., Any]: ...



from functools import cached_property
from typing import Any

from networkx.classes.coreviews import MultiAdjacencyView
from networkx.classes.digraph import DiGraph
from networkx.classes.graph import _Node
from networkx.classes.multigraph import MultiGraph
from networkx.classes.reportviews import (
    DiMultiDegreeView,
    InMultiDegreeView,
    InMultiEdgeView,
    OutMultiDegreeView,
    OutMultiEdgeView,
)

__all__ = ["MultiDiGraph"]

# NOTE: Graph subclasses relationships are so complex
# we're only overriding methods that differ in signature from the base classes
# to use inheritance to our advantage and reduce complexity
class MultiDiGraph(MultiGraph[_Node], DiGraph[_Node]):
    @cached_property
    def succ(self) -> MultiAdjacencyView[_Node, _Node, dict[str, Any]]: ...
    @cached_property
    def pred(self) -> MultiAdjacencyView[_Node, _Node, dict[str, Any]]: ...
    @cached_property
    def edges(self) -> OutMultiEdgeView[_Node]: ...
    @cached_property
    def out_edges(self) -> OutMultiEdgeView[_Node]: ...
    @cached_property
    def in_edges(self) -> InMultiEdgeView[_Node]: ...
    @cached_property
    def degree(self) -> DiMultiDegreeView[_Node]: ...
    @cached_property
    def in_degree(self) -> InMultiDegreeView[_Node]: ...
    @cached_property
    def out_degree(self) -> OutMultiDegreeView[_Node]: ...
    def to_undirected(self, reciprocal: bool = False, as_view: bool = False) -> MultiGraph[_Node]: ...  # type: ignore[override] # Has an additional `reciprocal` keyword argument



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import sys
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import tensorflow.core.framework.cost_graph_pb2
import tensorflow.core.framework.graph_pb2
import tensorflow.core.framework.step_stats_pb2
import tensorflow.core.protobuf.cluster_pb2
import tensorflow.core.protobuf.debug_pb2
import tensorflow.core.protobuf.rewriter_config_pb2
import tensorflow.tsl.protobuf.coordination_config_pb2
import tensorflow.tsl.protobuf.rpc_options_pb2

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class GPUOptions(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class Experimental(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        @typing.final
        class VirtualDevices(google.protobuf.message.Message):
            """Configuration for breaking down a visible GPU into multiple "virtual"
            devices.
            """

            DESCRIPTOR: google.protobuf.descriptor.Descriptor

            MEMORY_LIMIT_MB_FIELD_NUMBER: builtins.int
            PRIORITY_FIELD_NUMBER: builtins.int
            DEVICE_ORDINAL_FIELD_NUMBER: builtins.int
            @property
            def memory_limit_mb(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]:
                """Per "virtual" device memory limit, in MB. The number of elements in
                the list is the number of virtual devices to create on the
                corresponding visible GPU (see "virtual_devices" below).
                If empty and `num_virtual_devices_per_gpu` is not set, it will create
                single virtual device taking all available memory from the device.

                For the concept of "visible" and "virtual" GPU, see the comments for
                "visible_device_list" above for more information.
                """

            @property
            def priority(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
                """Priority values to use with the virtual devices. Use the cuda function
                cudaDeviceGetStreamPriorityRange to query for valid range of values for
                priority.

                On a P4000 GPU with cuda 10.1, the priority range reported was 0 for
                least priority and -1 for greatest priority.

                If this field is not specified, then the virtual devices will be
                created with the default. If this field has values set, then the size
                of this must match with the above memory_limit_mb.
                """

            @property
            def device_ordinal(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
                """Virtual Device ordinal number determines the device ID of the device.
                A Virtual device with a lower ordinal number always receives the a
                smaller device id. The phyiscal device id and location in the
                virtual device list is used to break ties.
                """

            def __init__(
                self,
                *,
                memory_limit_mb: collections.abc.Iterable[builtins.float] | None = ...,
                priority: collections.abc.Iterable[builtins.int] | None = ...,
                device_ordinal: collections.abc.Iterable[builtins.int] | None = ...,
            ) -> None: ...
            def ClearField(
                self,
                field_name: typing.Literal[
                    "device_ordinal", b"device_ordinal", "memory_limit_mb", b"memory_limit_mb", "priority", b"priority"
                ],
            ) -> None: ...

        @typing.final
        class StreamMergeOptions(google.protobuf.message.Message):
            """Whether to merge data transfer streams into the compute stream in the
            same stream group. Stream merging helps reduce the overhead caused by
            stream synchronization, especially when data transfers are frequent. For
            example, setting "merge_host_to_device_stream = true" will make the
            compute stream responsible for both computation and host to device memory
            copy.
            """

            DESCRIPTOR: google.protobuf.descriptor.Descriptor

            MERGE_HOST_TO_DEVICE_STREAM_FIELD_NUMBER: builtins.int
            MERGE_DEVICE_TO_HOST_STREAM_FIELD_NUMBER: builtins.int
            MERGE_DEVICE_TO_DEVICE_STREAM_FIELD_NUMBER: builtins.int
            merge_host_to_device_stream: builtins.bool
            """If true, the compute stream will be used for host_to_device copy as
            well. It's no longer necessary to record an event before the copy to
            let the copy stream wait for the compute stream to finish. There is
            also no need to wait for the copy to complete before executing the
            callback function.
            """
            merge_device_to_host_stream: builtins.bool
            """If true, the compute stream will be used for device_to_host copy as
            well. It's no longer necessary to record an event before the copy to
            let the copy stream wait for the compute stream to finish.
            """
            merge_device_to_device_stream: builtins.bool
            """If true, the compute stream will be used for device_to_device copy as
            well. It's no longer necessary to record an event before the copy to
            let the copy stream wait for the compute stream of the sending device
            to finish. There is also no need to wait for the compute stream of the
            receiving device to finish if the copy is within the same device.
            """
            def __init__(
                self,
                *,
                merge_host_to_device_stream: builtins.bool | None = ...,
                merge_device_to_host_stream: builtins.bool | None = ...,
                merge_device_to_device_stream: builtins.bool | None = ...,
            ) -> None: ...
            def ClearField(
                self,
                field_name: typing.Literal[
                    "merge_device_to_device_stream",
                    b"merge_device_to_device_stream",
                    "merge_device_to_host_stream",
                    b"merge_device_to_host_stream",
                    "merge_host_to_device_stream",
                    b"merge_host_to_device_stream",
                ],
            ) -> None: ...

        VIRTUAL_DEVICES_FIELD_NUMBER: builtins.int
        NUM_VIRTUAL_DEVICES_PER_GPU_FIELD_NUMBER: builtins.int
        USE_UNIFIED_MEMORY_FIELD_NUMBER: builtins.int
        NUM_DEV_TO_DEV_COPY_STREAMS_FIELD_NUMBER: builtins.int
        COLLECTIVE_RING_ORDER_FIELD_NUMBER: builtins.int
        TIMESTAMPED_ALLOCATOR_FIELD_NUMBER: builtins.int
        KERNEL_TRACKER_MAX_INTERVAL_FIELD_NUMBER: builtins.int
        KERNEL_TRACKER_MAX_BYTES_FIELD_NUMBER: builtins.int
        KERNEL_TRACKER_MAX_PENDING_FIELD_NUMBER: builtins.int
        INTERNAL_FRAGMENTATION_FRACTION_FIELD_NUMBER: builtins.int
        USE_CUDA_MALLOC_ASYNC_FIELD_NUMBER: builtins.int
        DISALLOW_RETRY_ON_ALLOCATION_FAILURE_FIELD_NUMBER: builtins.int
        GPU_HOST_MEM_LIMIT_IN_MB_FIELD_NUMBER: builtins.int
        GPU_HOST_MEM_DISALLOW_GROWTH_FIELD_NUMBER: builtins.int
        GPU_SYSTEM_MEMORY_SIZE_IN_MB_FIELD_NUMBER: builtins.int
        POPULATE_PJRT_GPU_CLIENT_CREATION_INFO_FIELD_NUMBER: builtins.int
        NODE_ID_FIELD_NUMBER: builtins.int
        STREAM_MERGE_OPTIONS_FIELD_NUMBER: builtins.int
        num_virtual_devices_per_gpu: builtins.int
        """The number of virtual devices to create on each visible GPU. The
        available memory will be split equally among all virtual devices. If the
        field `memory_limit_mb` in `VirtualDevices` is not empty, this field will
        be ignored.
        """
        use_unified_memory: builtins.bool
        """If true, uses CUDA unified memory for memory allocations. If
        per_process_gpu_memory_fraction option is greater than 1.0, then unified
        memory is used regardless of the value for this field. See comments for
        per_process_gpu_memory_fraction field for more details and requirements
        of the unified memory. This option is useful to oversubscribe memory if
        multiple processes are sharing a single GPU while individually using less
        than 1.0 per process memory fraction.
        """
        num_dev_to_dev_copy_streams: builtins.int
        """If > 1, the number of device-to-device copy streams to create
        for each GPUDevice.  Default value is 0, which is automatically
        converted to 1.
        """
        collective_ring_order: builtins.str
        """If non-empty, defines a good GPU ring order on a single worker based on
        device interconnect.  This assumes that all workers have the same GPU
        topology.  Specify as a comma-separated string, e.g. "3,2,1,0,7,6,5,4".
        This ring order is used by the RingReducer implementation of
        CollectiveReduce, and serves as an override to automatic ring order
        generation in OrderTaskDeviceMap() during CollectiveParam resolution.
        """
        timestamped_allocator: builtins.bool
        """If true then extra work is done by GPUDevice and GPUBFCAllocator to
        keep track of when GPU memory is freed and when kernels actually
        complete so that we can know when a nominally free memory chunk
        is really not subject to pending use.
        """
        kernel_tracker_max_interval: builtins.int
        """reserved id: 6

        Parameters for GPUKernelTracker.  By default no kernel tracking is done.
        Note that timestamped_allocator is only effective if some tracking is
        specified.

        If kernel_tracker_max_interval = n > 0, then a tracking event
        is inserted after every n kernels without an event.
        """
        kernel_tracker_max_bytes: builtins.int
        """If kernel_tracker_max_bytes = n > 0, then a tracking event is
        inserted after every series of kernels allocating a sum of
        memory >= n.  If one kernel allocates b * n bytes, then one
        event will be inserted after it, but it will count as b against
        the pending limit.
        """
        kernel_tracker_max_pending: builtins.int
        """If kernel_tracker_max_pending > 0 then no more than this many
        tracking events can be outstanding at a time.  An attempt to
        launch an additional kernel will stall until an event
        completes.
        """
        internal_fragmentation_fraction: builtins.float
        """BFC Allocator can return an allocated chunk of memory upto 2x the
        requested size. For virtual devices with tight memory constraints, and
        proportionately large allocation requests, this can lead to a significant
        reduction in available memory. The threshold below controls when a chunk
        should be split if the chunk size exceeds requested memory size. It is
        expressed as a fraction of total available memory for the tf device. For
        example setting it to 0.05 would imply a chunk needs to be split if its
        size exceeds the requested memory by 5% of the total virtual device/gpu
        memory size.
        """
        use_cuda_malloc_async: builtins.bool
        """When true, use CUDA cudaMallocAsync API instead of TF gpu allocator."""
        disallow_retry_on_allocation_failure: builtins.bool
        """By default, BFCAllocator may sleep when it runs out of memory, in the
        hopes that another thread will free up memory in the meantime.  Setting
        this to true disables the sleep; instead we'll OOM immediately.
        """
        gpu_host_mem_limit_in_mb: builtins.float
        """Memory limit for "GPU host allocator", aka pinned memory allocator.  This
        can also be set via the envvar TF_GPU_HOST_MEM_LIMIT_IN_MB.
        """
        gpu_host_mem_disallow_growth: builtins.bool
        """If true, then the host allocator allocates its max memory all upfront and
        never grows.  This can be useful for latency-sensitive systems, because
        growing the GPU host memory pool can be expensive.

        You probably only want to use this in combination with
        gpu_host_mem_limit_in_mb, because the default GPU host memory limit is
        quite high.
        """
        gpu_system_memory_size_in_mb: builtins.int
        """Memory limit for gpu system. This can also be set by
        TF_DEVICE_MIN_SYS_MEMORY_IN_MB, which takes precedence over
        gpu_system_memory_size_in_mb. With this, user can configure the gpu
        system memory size for better resource estimation of multi-tenancy(one
        gpu with multiple model) use case.
        """
        populate_pjrt_gpu_client_creation_info: builtins.bool
        """If true, save information needed for created a PjRt GPU client for
        creating a client with remote devices.
        """
        node_id: builtins.int
        """node_id for use when creating a PjRt GPU client with remote devices,
        which enumerates jobs*tasks from a ServerDef.
        """
        @property
        def virtual_devices(
            self,
        ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___GPUOptions.Experimental.VirtualDevices]:
            """The multi virtual device settings. If empty (not set), it will create
            single virtual device on each visible GPU, according to the settings
            in "visible_device_list" above. Otherwise, the number of elements in the
            list must be the same as the number of visible GPUs (after
            "visible_device_list" filtering if it is set), and the string represented
            device names (e.g. /device:GPU:<id>) will refer to the virtual
            devices and have the <id> field assigned sequentially starting from 0,
            according to the order of the virtual devices determined by
            device_ordinal and the location in the virtual device list.

            For example,
              visible_device_list = "1,0"
              virtual_devices { memory_limit: 1GB memory_limit: 2GB }
              virtual_devices { memory_limit: 3GB memory_limit: 4GB }
            will create 4 virtual devices as:
              /device:GPU:0 -> visible GPU 1 with 1GB memory
              /device:GPU:1 -> visible GPU 1 with 2GB memory
              /device:GPU:2 -> visible GPU 0 with 3GB memory
              /device:GPU:3 -> visible GPU 0 with 4GB memory

            but
              visible_device_list = "1,0"
              virtual_devices { memory_limit: 1GB memory_limit: 2GB
                                device_ordinal: 10 device_ordinal: 20}
              virtual_devices { memory_limit: 3GB memory_limit: 4GB
                                device_ordinal: 10 device_ordinal: 20}
            will create 4 virtual devices as:
              /device:GPU:0 -> visible GPU 1 with 1GB memory  (ordinal 10)
              /device:GPU:1 -> visible GPU 0 with 3GB memory  (ordinal 10)
              /device:GPU:2 -> visible GPU 1 with 2GB memory  (ordinal 20)
              /device:GPU:3 -> visible GPU 0 with 4GB memory  (ordinal 20)

            NOTE:
            1. It's invalid to set both this and "per_process_gpu_memory_fraction"
               at the same time.
            2. Currently this setting is per-process, not per-session. Using
               different settings in different sessions within same process will
               result in undefined behavior.
            """

        @property
        def stream_merge_options(self) -> global___GPUOptions.Experimental.StreamMergeOptions: ...
        def __init__(
            self,
            *,
            virtual_devices: collections.abc.Iterable[global___GPUOptions.Experimental.VirtualDevices] | None = ...,
            num_virtual_devices_per_gpu: builtins.int | None = ...,
            use_unified_memory: builtins.bool | None = ...,
            num_dev_to_dev_copy_streams: builtins.int | None = ...,
            collective_ring_order: builtins.str | None = ...,
            timestamped_allocator: builtins.bool | None = ...,
            kernel_tracker_max_interval: builtins.int | None = ...,
            kernel_tracker_max_bytes: builtins.int | None = ...,
            kernel_tracker_max_pending: builtins.int | None = ...,
            internal_fragmentation_fraction: builtins.float | None = ...,
            use_cuda_malloc_async: builtins.bool | None = ...,
            disallow_retry_on_allocation_failure: builtins.bool | None = ...,
            gpu_host_mem_limit_in_mb: builtins.float | None = ...,
            gpu_host_mem_disallow_growth: builtins.bool | None = ...,
            gpu_system_memory_size_in_mb: builtins.int | None = ...,
            populate_pjrt_gpu_client_creation_info: builtins.bool | None = ...,
            node_id: builtins.int | None = ...,
            stream_merge_options: global___GPUOptions.Experimental.StreamMergeOptions | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["stream_merge_options", b"stream_merge_options"]) -> builtins.bool: ...
        def ClearField(
            self,
            field_name: typing.Literal[
                "collective_ring_order",
                b"collective_ring_order",
                "disallow_retry_on_allocation_failure",
                b"disallow_retry_on_allocation_failure",
                "gpu_host_mem_disallow_growth",
                b"gpu_host_mem_disallow_growth",
                "gpu_host_mem_limit_in_mb",
                b"gpu_host_mem_limit_in_mb",
                "gpu_system_memory_size_in_mb",
                b"gpu_system_memory_size_in_mb",
                "internal_fragmentation_fraction",
                b"internal_fragmentation_fraction",
                "kernel_tracker_max_bytes",
                b"kernel_tracker_max_bytes",
                "kernel_tracker_max_interval",
                b"kernel_tracker_max_interval",
                "kernel_tracker_max_pending",
                b"kernel_tracker_max_pending",
                "node_id",
                b"node_id",
                "num_dev_to_dev_copy_streams",
                b"num_dev_to_dev_copy_streams",
                "num_virtual_devices_per_gpu",
                b"num_virtual_devices_per_gpu",
                "populate_pjrt_gpu_client_creation_info",
                b"populate_pjrt_gpu_client_creation_info",
                "stream_merge_options",
                b"stream_merge_options",
                "timestamped_allocator",
                b"timestamped_allocator",
                "use_cuda_malloc_async",
                b"use_cuda_malloc_async",
                "use_unified_memory",
                b"use_unified_memory",
                "virtual_devices",
                b"virtual_devices",
            ],
        ) -> None: ...

    PER_PROCESS_GPU_MEMORY_FRACTION_FIELD_NUMBER: builtins.int
    ALLOW_GROWTH_FIELD_NUMBER: builtins.int
    ALLOCATOR_TYPE_FIELD_NUMBER: builtins.int
    DEFERRED_DELETION_BYTES_FIELD_NUMBER: builtins.int
    VISIBLE_DEVICE_LIST_FIELD_NUMBER: builtins.int
    POLLING_ACTIVE_DELAY_USECS_FIELD_NUMBER: builtins.int
    POLLING_INACTIVE_DELAY_MSECS_FIELD_NUMBER: builtins.int
    FORCE_GPU_COMPATIBLE_FIELD_NUMBER: builtins.int
    EXPERIMENTAL_FIELD_NUMBER: builtins.int
    per_process_gpu_memory_fraction: builtins.float
    """Fraction of the total GPU memory to allocate for each process.
    1 means to allocate all of the GPU memory, 0.5 means the process
    allocates up to ~50% of the total GPU memory.

    GPU memory is pre-allocated unless the allow_growth option is enabled.

    If greater than 1.0, uses CUDA unified memory to potentially oversubscribe
    the amount of memory available on the GPU device by using host memory as a
    swap space. Accessing memory not available on the device will be
    significantly slower as that would require memory transfer between the host
    and the device. Options to reduce the memory requirement should be
    considered before enabling this option as this may come with a negative
    performance impact. Oversubscription using the unified memory requires
    Pascal class or newer GPUs and it is currently only supported on the Linux
    operating system. See
    https://docs.nvidia.com/cuda/cuda-c-programming-guide/index.html#um-requirements
    for the detailed requirements.
    """
    allow_growth: builtins.bool
    """If true, the allocator does not pre-allocate the entire specified
    GPU memory region, instead starting small and growing as needed.
    """
    allocator_type: builtins.str
    """The type of GPU allocation strategy to use.

    Allowed values:
    "": The empty string (default) uses a system-chosen default
        which may change over time.

    "BFC": A "Best-fit with coalescing" algorithm, simplified from a
           version of dlmalloc.
    """
    deferred_deletion_bytes: builtins.int
    """Delay deletion of up to this many bytes to reduce the number of
    interactions with gpu driver code.  If 0, the system chooses
    a reasonable default (several MBs).
    """
    visible_device_list: builtins.str
    """A comma-separated list of GPU ids that determines the 'visible'
    to 'virtual' mapping of GPU devices.  For example, if TensorFlow
    can see 8 GPU devices in the process, and one wanted to map
    visible GPU devices 5 and 3 as "/device:GPU:0", and "/device:GPU:1",
    then one would specify this field as "5,3".  This field is similar in
    spirit to the CUDA_VISIBLE_DEVICES environment variable, except
    it applies to the visible GPU devices in the process.

    NOTE:
    1. The GPU driver provides the process with the visible GPUs
       in an order which is not guaranteed to have any correlation to
       the *physical* GPU id in the machine.  This field is used for
       remapping "visible" to "virtual", which means this operates only
       after the process starts.  Users are required to use vendor
       specific mechanisms (e.g., CUDA_VISIBLE_DEVICES) to control the
       physical to visible device mapping prior to invoking TensorFlow.
    2. In the code, the ids in this list are also called "platform GPU id"s,
       and the 'virtual' ids of GPU devices (i.e. the ids in the device
       name "/device:GPU:<id>") are also called "TF GPU id"s. Please
       refer to third_party/tensorflow/core/common_runtime/gpu/gpu_id.h
       for more information.
    """
    polling_active_delay_usecs: builtins.int
    """In the event polling loop sleep this many microseconds between
    PollEvents calls, when the queue is not empty.  If value is not
    set or set to 0, gets set to a non-zero default.
    """
    polling_inactive_delay_msecs: builtins.int
    """This field is deprecated and ignored."""
    force_gpu_compatible: builtins.bool
    """Force all tensors to be gpu_compatible. On a GPU-enabled TensorFlow,
    enabling this option forces all CPU tensors to be allocated with Cuda
    pinned memory. Normally, TensorFlow will infer which tensors should be
    allocated as the pinned memory. But in case where the inference is
    incomplete, this option can significantly speed up the cross-device memory
    copy performance as long as it fits the memory.
    Note that this option is not something that should be
    enabled by default for unknown or very large models, since all Cuda pinned
    memory is unpageable, having too much pinned memory might negatively impact
    the overall host system performance.
    """
    @property
    def experimental(self) -> global___GPUOptions.Experimental:
        """Everything inside experimental is subject to change and is not subject
        to API stability guarantees in
        https://www.tensorflow.org/guide/version_compat.
        """

    def __init__(
        self,
        *,
        per_process_gpu_memory_fraction: builtins.float | None = ...,
        allow_growth: builtins.bool | None = ...,
        allocator_type: builtins.str | None = ...,
        deferred_deletion_bytes: builtins.int | None = ...,
        visible_device_list: builtins.str | None = ...,
        polling_active_delay_usecs: builtins.int | None = ...,
        polling_inactive_delay_msecs: builtins.int | None = ...,
        force_gpu_compatible: builtins.bool | None = ...,
        experimental: global___GPUOptions.Experimental | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["experimental", b"experimental"]) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "allocator_type",
            b"allocator_type",
            "allow_growth",
            b"allow_growth",
            "deferred_deletion_bytes",
            b"deferred_deletion_bytes",
            "experimental",
            b"experimental",
            "force_gpu_compatible",
            b"force_gpu_compatible",
            "per_process_gpu_memory_fraction",
            b"per_process_gpu_memory_fraction",
            "polling_active_delay_usecs",
            b"polling_active_delay_usecs",
            "polling_inactive_delay_msecs",
            b"polling_inactive_delay_msecs",
            "visible_device_list",
            b"visible_device_list",
        ],
    ) -> None: ...

global___GPUOptions = GPUOptions

@typing.final
class OptimizerOptions(google.protobuf.message.Message):
    """Options passed to the graph optimizer"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _Level:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _LevelEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[OptimizerOptions._Level.ValueType], builtins.type
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        L1: OptimizerOptions._Level.ValueType  # 0
        """L1 is the default level.
        Optimization performed at L1 :
        1. Common subexpression elimination
        2. Constant folding
        """
        L0: OptimizerOptions._Level.ValueType  # -1
        """No optimizations"""

    class Level(_Level, metaclass=_LevelEnumTypeWrapper):
        """Optimization level"""

    L1: OptimizerOptions.Level.ValueType  # 0
    """L1 is the default level.
    Optimization performed at L1 :
    1. Common subexpression elimination
    2. Constant folding
    """
    L0: OptimizerOptions.Level.ValueType  # -1
    """No optimizations"""

    class _GlobalJitLevel:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _GlobalJitLevelEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[OptimizerOptions._GlobalJitLevel.ValueType], builtins.type
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        DEFAULT: OptimizerOptions._GlobalJitLevel.ValueType  # 0
        """Default setting ("off" now, but later expected to be "on")"""
        OFF: OptimizerOptions._GlobalJitLevel.ValueType  # -1
        ON_1: OptimizerOptions._GlobalJitLevel.ValueType  # 1
        """The following settings turn on compilation, with higher values being
        more aggressive.  Higher values may reduce opportunities for parallelism
        and may use more memory.  (At present, there is no distinction, but this
        is expected to change.)
        """
        ON_2: OptimizerOptions._GlobalJitLevel.ValueType  # 2

    class GlobalJitLevel(_GlobalJitLevel, metaclass=_GlobalJitLevelEnumTypeWrapper):
        """Control the use of the compiler/jit.  Experimental."""

    DEFAULT: OptimizerOptions.GlobalJitLevel.ValueType  # 0
    """Default setting ("off" now, but later expected to be "on")"""
    OFF: OptimizerOptions.GlobalJitLevel.ValueType  # -1
    ON_1: OptimizerOptions.GlobalJitLevel.ValueType  # 1
    """The following settings turn on compilation, with higher values being
    more aggressive.  Higher values may reduce opportunities for parallelism
    and may use more memory.  (At present, there is no distinction, but this
    is expected to change.)
    """
    ON_2: OptimizerOptions.GlobalJitLevel.ValueType  # 2

    DO_COMMON_SUBEXPRESSION_ELIMINATION_FIELD_NUMBER: builtins.int
    DO_CONSTANT_FOLDING_FIELD_NUMBER: builtins.int
    MAX_FOLDED_CONSTANT_IN_BYTES_FIELD_NUMBER: builtins.int
    DO_FUNCTION_INLINING_FIELD_NUMBER: builtins.int
    OPT_LEVEL_FIELD_NUMBER: builtins.int
    GLOBAL_JIT_LEVEL_FIELD_NUMBER: builtins.int
    CPU_GLOBAL_JIT_FIELD_NUMBER: builtins.int
    do_common_subexpression_elimination: builtins.bool
    """If true, optimize the graph using common subexpression elimination.
    Note: the optimization Level L1 will override this setting to true. So in
    order to disable common subexpression elimination the opt_level has to be
    set to L0.
    """
    do_constant_folding: builtins.bool
    """If true, perform constant folding optimization on the graph.
    Note: the optimization Level L1 will override this setting to true. So in
    order to disable constant folding the opt_level has to be set to L0.
    """
    max_folded_constant_in_bytes: builtins.int
    """Constant folding optimization replaces tensors whose values can be
    predetermined, with constant nodes. To avoid inserting too large constants,
    the size of each constant created can be limited. If this value is zero, a
    default limit of 10 MiB will be applied. If constant folding optimization
    is disabled, this value is ignored.
    """
    do_function_inlining: builtins.bool
    """If true, perform function inlining on the graph."""
    opt_level: global___OptimizerOptions.Level.ValueType
    """Overall optimization level. The actual optimizations applied will be the
    logical OR of the flags that this level implies and any flags already set.
    """
    global_jit_level: global___OptimizerOptions.GlobalJitLevel.ValueType
    cpu_global_jit: builtins.bool
    """CPU code will be autoclustered only if global_jit_level >= ON_1 and either:
     - this flag is true, or
     - TF_XLA_FLAGS contains --tf_xla_cpu_global_jit=true.
    """
    def __init__(
        self,
        *,
        do_common_subexpression_elimination: builtins.bool | None = ...,
        do_constant_folding: builtins.bool | None = ...,
        max_folded_constant_in_bytes: builtins.int | None = ...,
        do_function_inlining: builtins.bool | None = ...,
        opt_level: global___OptimizerOptions.Level.ValueType | None = ...,
        global_jit_level: global___OptimizerOptions.GlobalJitLevel.ValueType | None = ...,
        cpu_global_jit: builtins.bool | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "cpu_global_jit",
            b"cpu_global_jit",
            "do_common_subexpression_elimination",
            b"do_common_subexpression_elimination",
            "do_constant_folding",
            b"do_constant_folding",
            "do_function_inlining",
            b"do_function_inlining",
            "global_jit_level",
            b"global_jit_level",
            "max_folded_constant_in_bytes",
            b"max_folded_constant_in_bytes",
            "opt_level",
            b"opt_level",
        ],
    ) -> None: ...

global___OptimizerOptions = OptimizerOptions

@typing.final
class GraphOptions(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ENABLE_RECV_SCHEDULING_FIELD_NUMBER: builtins.int
    OPTIMIZER_OPTIONS_FIELD_NUMBER: builtins.int
    BUILD_COST_MODEL_FIELD_NUMBER: builtins.int
    BUILD_COST_MODEL_AFTER_FIELD_NUMBER: builtins.int
    INFER_SHAPES_FIELD_NUMBER: builtins.int
    PLACE_PRUNED_GRAPH_FIELD_NUMBER: builtins.int
    ENABLE_BFLOAT16_SENDRECV_FIELD_NUMBER: builtins.int
    TIMELINE_STEP_FIELD_NUMBER: builtins.int
    REWRITE_OPTIONS_FIELD_NUMBER: builtins.int
    enable_recv_scheduling: builtins.bool
    """If true, use control flow to schedule the activation of Recv nodes.
    (Currently ignored.)
    """
    build_cost_model: builtins.int
    """The number of steps to run before returning a cost model detailing
    the memory usage and performance of each node of the graph. 0 means
    no cost model.
    """
    build_cost_model_after: builtins.int
    """The number of steps to skip before collecting statistics for the
    cost model.
    """
    infer_shapes: builtins.bool
    """Annotate each Node with Op output shape data, to the extent it can
    be statically inferred.
    """
    place_pruned_graph: builtins.bool
    """Only place the subgraphs that are run, rather than the entire graph.

    This is useful for interactive graph building, where one might
    produce graphs that cannot be placed during the debugging
    process.  In particular, it allows the client to continue work in
    a session after adding a node to a graph whose placement
    constraints are unsatisfiable.
    """
    enable_bfloat16_sendrecv: builtins.bool
    """If true, transfer float values between processes as bfloat16."""
    timeline_step: builtins.int
    """If > 0, record a timeline every this many steps.
    EXPERIMENTAL: This currently has no effect in MasterSession.
    """
    @property
    def optimizer_options(self) -> global___OptimizerOptions:
        """Options controlling how graph is optimized."""

    @property
    def rewrite_options(self) -> tensorflow.core.protobuf.rewriter_config_pb2.RewriterConfig:
        """Options that control the type and amount of graph rewriting.
        Not currently configurable via the public Python API (i.e. there is no API
        stability guarantee if you import RewriterConfig explicitly).
        """

    def __init__(
        self,
        *,
        enable_recv_scheduling: builtins.bool | None = ...,
        optimizer_options: global___OptimizerOptions | None = ...,
        build_cost_model: builtins.int | None = ...,
        build_cost_model_after: builtins.int | None = ...,
        infer_shapes: builtins.bool | None = ...,
        place_pruned_graph: builtins.bool | None = ...,
        enable_bfloat16_sendrecv: builtins.bool | None = ...,
        timeline_step: builtins.int | None = ...,
        rewrite_options: tensorflow.core.protobuf.rewriter_config_pb2.RewriterConfig | None = ...,
    ) -> None: ...
    def HasField(
        self, field_name: typing.Literal["optimizer_options", b"optimizer_options", "rewrite_options", b"rewrite_options"]
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "build_cost_model",
            b"build_cost_model",
            "build_cost_model_after",
            b"build_cost_model_after",
            "enable_bfloat16_sendrecv",
            b"enable_bfloat16_sendrecv",
            "enable_recv_scheduling",
            b"enable_recv_scheduling",
            "infer_shapes",
            b"infer_shapes",
            "optimizer_options",
            b"optimizer_options",
            "place_pruned_graph",
            b"place_pruned_graph",
            "rewrite_options",
            b"rewrite_options",
            "timeline_step",
            b"timeline_step",
        ],
    ) -> None: ...

global___GraphOptions = GraphOptions

@typing.final
class ThreadPoolOptionProto(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NUM_THREADS_FIELD_NUMBER: builtins.int
    GLOBAL_NAME_FIELD_NUMBER: builtins.int
    num_threads: builtins.int
    """The number of threads in the pool.

    0 means the system picks a value based on where this option proto is used
    (see the declaration of the specific field for more info).
    """
    global_name: builtins.str
    """The global name of the threadpool.

    If empty, then the threadpool is made and used according to the scope it's
    in - e.g., for a session threadpool, it is used by that session only.

    If non-empty, then:
    - a global threadpool associated with this name is looked
      up or created. This allows, for example, sharing one threadpool across
      many sessions (e.g., like the default behavior, if
      inter_op_parallelism_threads is not configured), but still partitioning
      into a large and small pool.
    - if the threadpool for this global_name already exists, then it is an
      error if the existing pool was created using a different num_threads
      value as is specified on this call.
    - threadpools created this way are never garbage collected.
    """
    def __init__(self, *, num_threads: builtins.int | None = ..., global_name: builtins.str | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["global_name", b"global_name", "num_threads", b"num_threads"]) -> None: ...

global___ThreadPoolOptionProto = ThreadPoolOptionProto

@typing.final
class SessionMetadata(google.protobuf.message.Message):
    """Metadata about the session.

    This can be used by the runtime and the Ops for debugging, monitoring, etc.

    The (name, version) tuple is expected to be a unique identifier for
    sessions within the same process.

    NOTE: This is currently used and propagated only by the direct session.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    VERSION_FIELD_NUMBER: builtins.int
    name: builtins.str
    version: builtins.int
    """The version is optional. If set, needs to be >= 0."""
    def __init__(self, *, name: builtins.str | None = ..., version: builtins.int | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["name", b"name", "version", b"version"]) -> None: ...

global___SessionMetadata = SessionMetadata

@typing.final
class ConfigProto(google.protobuf.message.Message):
    """Session configuration parameters.
    The system picks appropriate values for fields that are not set.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class DeviceCountEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.int
        def __init__(self, *, key: builtins.str | None = ..., value: builtins.int | None = ...) -> None: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    @typing.final
    class Experimental(google.protobuf.message.Message):
        """Everything inside Experimental is subject to change and is not subject
        to API stability guarantees in
        https://www.tensorflow.org/guide/version_compat.
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        class _MlirBridgeRollout:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _MlirBridgeRolloutEnumTypeWrapper(
            google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[ConfigProto.Experimental._MlirBridgeRollout.ValueType],
            builtins.type,
        ):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            MLIR_BRIDGE_ROLLOUT_UNSPECIFIED: ConfigProto.Experimental._MlirBridgeRollout.ValueType  # 0
            """If this field is left unspecified, the MLIR bridge may be selectively
            enabled on a per graph basis.
            """
            MLIR_BRIDGE_ROLLOUT_ENABLED: ConfigProto.Experimental._MlirBridgeRollout.ValueType  # 1
            """Enabling the MLIR bridge enables it for all graphs in this session."""
            MLIR_BRIDGE_ROLLOUT_DISABLED: ConfigProto.Experimental._MlirBridgeRollout.ValueType  # 2
            """Disabling the MLIR bridge disables it for all graphs in this session."""

        class MlirBridgeRollout(_MlirBridgeRollout, metaclass=_MlirBridgeRolloutEnumTypeWrapper):
            """An enum that describes the state of the MLIR bridge rollout."""

        MLIR_BRIDGE_ROLLOUT_UNSPECIFIED: ConfigProto.Experimental.MlirBridgeRollout.ValueType  # 0
        """If this field is left unspecified, the MLIR bridge may be selectively
        enabled on a per graph basis.
        """
        MLIR_BRIDGE_ROLLOUT_ENABLED: ConfigProto.Experimental.MlirBridgeRollout.ValueType  # 1
        """Enabling the MLIR bridge enables it for all graphs in this session."""
        MLIR_BRIDGE_ROLLOUT_DISABLED: ConfigProto.Experimental.MlirBridgeRollout.ValueType  # 2
        """Disabling the MLIR bridge disables it for all graphs in this session."""

        COLLECTIVE_GROUP_LEADER_FIELD_NUMBER: builtins.int
        EXECUTOR_TYPE_FIELD_NUMBER: builtins.int
        RECV_BUF_MAX_CHUNK_FIELD_NUMBER: builtins.int
        USE_NUMA_AFFINITY_FIELD_NUMBER: builtins.int
        COLLECTIVE_DETERMINISTIC_SEQUENTIAL_EXECUTION_FIELD_NUMBER: builtins.int
        COLLECTIVE_NCCL_FIELD_NUMBER: builtins.int
        SHARE_SESSION_STATE_IN_CLUSTERSPEC_PROPAGATION_FIELD_NUMBER: builtins.int
        DISABLE_THREAD_SPINNING_FIELD_NUMBER: builtins.int
        SHARE_CLUSTER_DEVICES_IN_SESSION_FIELD_NUMBER: builtins.int
        SESSION_METADATA_FIELD_NUMBER: builtins.int
        OPTIMIZE_FOR_STATIC_GRAPH_FIELD_NUMBER: builtins.int
        ENABLE_MLIR_BRIDGE_FIELD_NUMBER: builtins.int
        MLIR_BRIDGE_ROLLOUT_FIELD_NUMBER: builtins.int
        ENABLE_MLIR_GRAPH_OPTIMIZATION_FIELD_NUMBER: builtins.int
        DISABLE_OUTPUT_PARTITION_GRAPHS_FIELD_NUMBER: builtins.int
        XLA_FUSION_AUTOTUNER_THRESH_FIELD_NUMBER: builtins.int
        USE_TFRT_FIELD_NUMBER: builtins.int
        ENABLE_MULTI_HOST_FIELD_NUMBER: builtins.int
        TFRT_USE_IFRT_FIELD_NUMBER: builtins.int
        BACKEND_SERVER_PORT_FIELD_NUMBER: builtins.int
        TARGET_TPU_FIELD_NUMBER: builtins.int
        TARGET_GPU_FIELD_NUMBER: builtins.int
        STREAM_MERGE_THRESHOLD_FIELD_NUMBER: builtins.int
        DISABLE_FUNCTIONAL_OPS_LOWERING_FIELD_NUMBER: builtins.int
        XLA_PREFER_SINGLE_GRAPH_CLUSTER_FIELD_NUMBER: builtins.int
        COORDINATION_CONFIG_FIELD_NUMBER: builtins.int
        DISABLE_OPTIMIZE_FOR_STATIC_GRAPH_FIELD_NUMBER: builtins.int
        DISABLE_EAGER_EXECUTOR_STREAMING_ENQUEUE_FIELD_NUMBER: builtins.int
        collective_group_leader: builtins.str
        """Task name for group resolution."""
        executor_type: builtins.str
        """Which executor to use, the default executor will be used
        if it is an empty string or "DEFAULT"
        """
        recv_buf_max_chunk: builtins.int
        """Guidance to formatting of large RecvBuf fields for transfer.
        Any positive value sets the max chunk size.  0 defaults to 4096.
        Any negative value indicates no max, i.e. one chunk only.
        """
        use_numa_affinity: builtins.bool
        """If true, and supported by the platform, the runtime will attempt to
        use NUMA affinity where applicable.  One consequence will be the
        existence of as many CPU devices as there are available NUMA nodes.
        """
        collective_deterministic_sequential_execution: builtins.bool
        """If true, make collective op execution order sequential and deterministic
        for potentially concurrent collective instances.
        """
        collective_nccl: builtins.bool
        """If true, use NCCL for CollectiveOps.  This feature is highly
        experimental.
        """
        share_session_state_in_clusterspec_propagation: builtins.bool
        """In the following, session state means the value of a variable, elements
        in a hash table, or any other resource, accessible by worker sessions
        held by a TF server.

        When ClusterSpec propagation is enabled, the value of
        isolate_session_state is ignored when deciding whether to share session
        states in a TF server (for backwards compatibility reasons).
        - If share_session_state_in_clusterspec_propagation is true, the session
        states are shared.
        - If share_session_state_in_clusterspec_propagation is false, session
        states are isolated.

        When clusterspec propagation is not used, the value of
        share_session_state_in_clusterspec_propagation is ignored when deciding
        whether to share session states in a TF server.
        - If isolate_session_state is true, session states are isolated.
        - If isolate_session_state is false, session states are shared.

        TODO(b/129330037): Add a single API that consistently treats
        isolate_session_state and ClusterSpec propagation.
        """
        disable_thread_spinning: builtins.bool
        """If using a direct session, disable spinning while waiting for work in
        the thread pool. This may result in higher latency for completing ops,
        but in the case where there is a lot of spinning may result in lower
        CPU usage.
        """
        share_cluster_devices_in_session: builtins.bool
        """This was promoted to a non-experimental API. Please use
        ConfigProto.share_cluster_devices_in_session instead.
        """
        optimize_for_static_graph: builtins.bool
        """If true, the session may treat the graph as being static for optimization
        purposes.

        If this option is set to true when a session is created, the full
        GraphDef must be passed in a single call to Session::Create(), and
        Session::Extend() may not be supported.
        """
        enable_mlir_bridge: builtins.bool
        """Whether to enable the MLIR-based TF->XLA bridge. This is only used if set
        to true. Default value or false is ignored. Use mlir_bridge_rollout for
        finer control.

        If this option is set to true when a session is created, MLIR is used to
        perform the set of graph transformations to put the graph in a form that
        can be executed with delegation of some computations to an accelerator.
        This builds on the model of XLA where a subset of the graph is
        encapsulated and attached to a "compile" operation, whose result is fed
        to an "execute" operation. The kernel for these operations is responsible
        to lower the encapsulated graph to a particular device.
        """
        mlir_bridge_rollout: global___ConfigProto.Experimental.MlirBridgeRollout.ValueType
        """Whether to enable the MLIR-based TF->XLA bridge."""
        enable_mlir_graph_optimization: builtins.bool
        """Whether to enable the MLIR-based Graph optimizations.

        This will become a part of standard Tensorflow graph optimization
        pipeline, currently this is only used for gradual migration and testing
        new passes that are replacing existing optimizations in Grappler.
        """
        disable_output_partition_graphs: builtins.bool
        """If true, the session will not store an additional copy of the graph for
        each subgraph.

        If this option is set to true when a session is created, the
        `RunOptions.output_partition_graphs` options must not be set.
        """
        xla_fusion_autotuner_thresh: builtins.int
        """Minimum number of batches run through the XLA graph before XLA fusion
        autotuner is enabled. Default value of zero disables the autotuner.

        The XLA fusion autotuner can improve performance by executing a heuristic
        search on the compiler parameters.
        """
        use_tfrt: builtins.bool
        """Whether runtime execution uses TFRT."""
        enable_multi_host: builtins.bool
        """If true, use Pathways with TFRT API for multi host support."""
        tfrt_use_ifrt: builtins.bool
        """If true, use ifrt as the backend for TFRT. This is only used when
        `use_tfrt` is true.
        """
        backend_server_port: builtins.int
        """Port for the Pathways server. Ignored if enable_multi_host=false."""
        target_tpu: builtins.bool
        """If true, TFRT will use TPU specific compiler passes and perform TPU
        specific initialization.
        """
        target_gpu: builtins.bool
        """If true, TFRT will use GPU specific compiler passes and perform GPU
        specific initialization.
        """
        stream_merge_threshold: builtins.int
        """The threshold to merge small streams in TFRT. The stream with cost
        smaller than the threshold will be merged. Setting it to value 1
        disables all merges.
        """
        disable_functional_ops_lowering: builtins.bool
        """Whether functional control flow op lowering should be disabled. This is
        useful when executing within a portable runtime where control flow op
        kernels may not be loaded due to selective registration.
        """
        xla_prefer_single_graph_cluster: builtins.bool
        """Provides a hint to XLA auto clustering to prefer forming a single large
        cluster that encompases most of the graph.
        """
        disable_optimize_for_static_graph: builtins.bool
        """If true, the session will treat the graph as being non-static for
        optimization purposes.

        If this option is set to true when a session is created, the full
        GraphDef will be retained to enable calls to Session::Extend().
        Calling Extend() without setting this flag will result in errors.

        This option is meant to replace `optimize_for_static_graph` and it
        aims to negate its value.
        """
        disable_eager_executor_streaming_enqueue: builtins.bool
        """Whether eager remote execution will stream all the function calls or
        allow them to happen in parallel. When true, streaming execution is
        disabled, and parallel execution is allowed.
        """
        @property
        def session_metadata(self) -> global___SessionMetadata:
            """Metadata about the session.

            If set, this can be used by the runtime and the Ops for debugging,
            monitoring, etc.

            NOTE: This is currently used and propagated only by the direct session
            and EagerContext.
            """

        @property
        def coordination_config(self) -> tensorflow.tsl.protobuf.coordination_config_pb2.CoordinationServiceConfig:
            """Distributed coordination service configurations."""

        def __init__(
            self,
            *,
            collective_group_leader: builtins.str | None = ...,
            executor_type: builtins.str | None = ...,
            recv_buf_max_chunk: builtins.int | None = ...,
            use_numa_affinity: builtins.bool | None = ...,
            collective_deterministic_sequential_execution: builtins.bool | None = ...,
            collective_nccl: builtins.bool | None = ...,
            share_session_state_in_clusterspec_propagation: builtins.bool | None = ...,
            disable_thread_spinning: builtins.bool | None = ...,
            share_cluster_devices_in_session: builtins.bool | None = ...,
            session_metadata: global___SessionMetadata | None = ...,
            optimize_for_static_graph: builtins.bool | None = ...,
            enable_mlir_bridge: builtins.bool | None = ...,
            mlir_bridge_rollout: global___ConfigProto.Experimental.MlirBridgeRollout.ValueType | None = ...,
            enable_mlir_graph_optimization: builtins.bool | None = ...,
            disable_output_partition_graphs: builtins.bool | None = ...,
            xla_fusion_autotuner_thresh: builtins.int | None = ...,
            use_tfrt: builtins.bool | None = ...,
            enable_multi_host: builtins.bool | None = ...,
            tfrt_use_ifrt: builtins.bool | None = ...,
            backend_server_port: builtins.int | None = ...,
            target_tpu: builtins.bool | None = ...,
            target_gpu: builtins.bool | None = ...,
            stream_merge_threshold: builtins.int | None = ...,
            disable_functional_ops_lowering: builtins.bool | None = ...,
            xla_prefer_single_graph_cluster: builtins.bool | None = ...,
            coordination_config: tensorflow.tsl.protobuf.coordination_config_pb2.CoordinationServiceConfig | None = ...,
            disable_optimize_for_static_graph: builtins.bool | None = ...,
            disable_eager_executor_streaming_enqueue: builtins.bool | None = ...,
        ) -> None: ...
        def HasField(
            self,
            field_name: typing.Literal["coordination_config", b"coordination_config", "session_metadata", b"session_metadata"],
        ) -> builtins.bool: ...
        def ClearField(
            self,
            field_name: typing.Literal[
                "backend_server_port",
                b"backend_server_port",
                "collective_deterministic_sequential_execution",
                b"collective_deterministic_sequential_execution",
                "collective_group_leader",
                b"collective_group_leader",
                "collective_nccl",
                b"collective_nccl",
                "coordination_config",
                b"coordination_config",
                "disable_eager_executor_streaming_enqueue",
                b"disable_eager_executor_streaming_enqueue",
                "disable_functional_ops_lowering",
                b"disable_functional_ops_lowering",
                "disable_optimize_for_static_graph",
                b"disable_optimize_for_static_graph",
                "disable_output_partition_graphs",
                b"disable_output_partition_graphs",
                "disable_thread_spinning",
                b"disable_thread_spinning",
                "enable_mlir_bridge",
                b"enable_mlir_bridge",
                "enable_mlir_graph_optimization",
                b"enable_mlir_graph_optimization",
                "enable_multi_host",
                b"enable_multi_host",
                "executor_type",
                b"executor_type",
                "mlir_bridge_rollout",
                b"mlir_bridge_rollout",
                "optimize_for_static_graph",
                b"optimize_for_static_graph",
                "recv_buf_max_chunk",
                b"recv_buf_max_chunk",
                "session_metadata",
                b"session_metadata",
                "share_cluster_devices_in_session",
                b"share_cluster_devices_in_session",
                "share_session_state_in_clusterspec_propagation",
                b"share_session_state_in_clusterspec_propagation",
                "stream_merge_threshold",
                b"stream_merge_threshold",
                "target_gpu",
                b"target_gpu",
                "target_tpu",
                b"target_tpu",
                "tfrt_use_ifrt",
                b"tfrt_use_ifrt",
                "use_numa_affinity",
                b"use_numa_affinity",
                "use_tfrt",
                b"use_tfrt",
                "xla_fusion_autotuner_thresh",
                b"xla_fusion_autotuner_thresh",
                "xla_prefer_single_graph_cluster",
                b"xla_prefer_single_graph_cluster",
            ],
        ) -> None: ...

    DEVICE_COUNT_FIELD_NUMBER: builtins.int
    INTRA_OP_PARALLELISM_THREADS_FIELD_NUMBER: builtins.int
    INTER_OP_PARALLELISM_THREADS_FIELD_NUMBER: builtins.int
    USE_PER_SESSION_THREADS_FIELD_NUMBER: builtins.int
    SESSION_INTER_OP_THREAD_POOL_FIELD_NUMBER: builtins.int
    PLACEMENT_PERIOD_FIELD_NUMBER: builtins.int
    DEVICE_FILTERS_FIELD_NUMBER: builtins.int
    GPU_OPTIONS_FIELD_NUMBER: builtins.int
    PLUGGABLE_DEVICE_OPTIONS_FIELD_NUMBER: builtins.int
    ALLOW_SOFT_PLACEMENT_FIELD_NUMBER: builtins.int
    LOG_DEVICE_PLACEMENT_FIELD_NUMBER: builtins.int
    GRAPH_OPTIONS_FIELD_NUMBER: builtins.int
    OPERATION_TIMEOUT_IN_MS_FIELD_NUMBER: builtins.int
    RPC_OPTIONS_FIELD_NUMBER: builtins.int
    CLUSTER_DEF_FIELD_NUMBER: builtins.int
    ISOLATE_SESSION_STATE_FIELD_NUMBER: builtins.int
    SHARE_CLUSTER_DEVICES_IN_SESSION_FIELD_NUMBER: builtins.int
    EXPERIMENTAL_FIELD_NUMBER: builtins.int
    intra_op_parallelism_threads: builtins.int
    """The execution of an individual op (for some op types) can be
    parallelized on a pool of intra_op_parallelism_threads.
    0 means the system picks an appropriate number.

    If you create an ordinary session, e.g., from Python or C++,
    then there is exactly one intra op thread pool per process.
    The first session created determines the number of threads in this pool.
    All subsequent sessions reuse/share this one global pool.

    There are notable exceptions to the default behavior described above:
    1. There is an environment variable  for overriding this thread pool,
       named TF_OVERRIDE_GLOBAL_THREADPOOL.
    2. When connecting to a server, such as a remote `tf.train.Server`
       instance, then this option will be ignored altogether.
    """
    inter_op_parallelism_threads: builtins.int
    """Nodes that perform blocking operations are enqueued on a pool of
    inter_op_parallelism_threads available in each process.

    0 means the system picks an appropriate number.
    Negative means all operations are performed in caller's thread.

    Note that the first Session created in the process sets the
    number of threads for all future sessions unless use_per_session_threads is
    true or session_inter_op_thread_pool is configured.
    """
    use_per_session_threads: builtins.bool
    """If true, use a new set of threads for this session rather than the global
    pool of threads. Only supported by direct sessions.

    If false, use the global threads created by the first session, or the
    per-session thread pools configured by session_inter_op_thread_pool.

    This option is deprecated. The same effect can be achieved by setting
    session_inter_op_thread_pool to have one element, whose num_threads equals
    inter_op_parallelism_threads.
    """
    placement_period: builtins.int
    """Assignment of Nodes to Devices is recomputed every placement_period
    steps until the system warms up (at which point the recomputation
    typically slows down automatically).
    """
    allow_soft_placement: builtins.bool
    """Whether soft placement is allowed. If allow_soft_placement is true,
    an op will be placed on CPU if
      1. there's no GPU implementation for the OP
    or
      2. no GPU devices are known or registered
    or
      3. need to co-locate with reftype input(s) which are from CPU.
    """
    log_device_placement: builtins.bool
    """Whether device placements should be logged."""
    operation_timeout_in_ms: builtins.int
    """Global timeout for all blocking operations in this session.  If non-zero,
    and not overridden on a per-operation basis, this value will be used as the
    deadline for all blocking operations.
    """
    isolate_session_state: builtins.bool
    """If true, any resources such as Variables used in the session will not be
    shared with other sessions. However, when clusterspec propagation is
    enabled, this field is ignored and sessions are always isolated.
    """
    share_cluster_devices_in_session: builtins.bool
    """When true, WorkerSessions are created with device attributes from the
    full cluster.
    This is helpful when a worker wants to partition a graph
    (for example during a PartitionedCallOp).
    """
    @property
    def device_count(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.int]:
        """Map from device type name (e.g., "CPU" or "GPU" ) to maximum
        number of devices of that type to use.  If a particular device
        type is not found in the map, the system picks an appropriate
        number.
        """

    @property
    def session_inter_op_thread_pool(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ThreadPoolOptionProto]:
        """This option is experimental - it may be replaced with a different mechanism
        in the future.

        Configures session thread pools. If this is configured, then RunOptions for
        a Run call can select the thread pool to use.

        The intended use is for when some session invocations need to run in a
        background pool limited to a small number of threads:
        - For example, a session may be configured to have one large pool (for
        regular compute) and one small pool (for periodic, low priority work);
        using the small pool is currently the mechanism for limiting the inter-op
        parallelism of the low priority work.  Note that it does not limit the
        parallelism of work spawned by a single op kernel implementation.
        - Using this setting is normally not needed in training, but may help some
        serving use cases.
        - It is also generally recommended to set the global_name field of this
        proto, to avoid creating multiple large pools. It is typically better to
        run the non-low-priority work, even across sessions, in a single large
        pool.
        """

    @property
    def device_filters(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """When any filters are present sessions will ignore all devices which do not
        match the filters. Each filter can be partially specified, e.g. "/job:ps"
        "/job:worker/replica:3", etc.
        """

    @property
    def gpu_options(self) -> global___GPUOptions:
        """Options that apply to all GPUs."""

    @property
    def pluggable_device_options(self) -> global___GPUOptions:
        """Options that apply to pluggable devices."""

    @property
    def graph_options(self) -> global___GraphOptions:
        """Options that apply to all graphs."""

    @property
    def rpc_options(self) -> tensorflow.tsl.protobuf.rpc_options_pb2.RPCOptions:
        """Options that apply when this session uses the distributed runtime."""

    @property
    def cluster_def(self) -> tensorflow.core.protobuf.cluster_pb2.ClusterDef:
        """Optional list of all workers to use in this session."""

    @property
    def experimental(self) -> global___ConfigProto.Experimental: ...
    def __init__(
        self,
        *,
        device_count: collections.abc.Mapping[builtins.str, builtins.int] | None = ...,
        intra_op_parallelism_threads: builtins.int | None = ...,
        inter_op_parallelism_threads: builtins.int | None = ...,
        use_per_session_threads: builtins.bool | None = ...,
        session_inter_op_thread_pool: collections.abc.Iterable[global___ThreadPoolOptionProto] | None = ...,
        placement_period: builtins.int | None = ...,
        device_filters: collections.abc.Iterable[builtins.str] | None = ...,
        gpu_options: global___GPUOptions | None = ...,
        pluggable_device_options: global___GPUOptions | None = ...,
        allow_soft_placement: builtins.bool | None = ...,
        log_device_placement: builtins.bool | None = ...,
        graph_options: global___GraphOptions | None = ...,
        operation_timeout_in_ms: builtins.int | None = ...,
        rpc_options: tensorflow.tsl.protobuf.rpc_options_pb2.RPCOptions | None = ...,
        cluster_def: tensorflow.core.protobuf.cluster_pb2.ClusterDef | None = ...,
        isolate_session_state: builtins.bool | None = ...,
        share_cluster_devices_in_session: builtins.bool | None = ...,
        experimental: global___ConfigProto.Experimental | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "cluster_def",
            b"cluster_def",
            "experimental",
            b"experimental",
            "gpu_options",
            b"gpu_options",
            "graph_options",
            b"graph_options",
            "pluggable_device_options",
            b"pluggable_device_options",
            "rpc_options",
            b"rpc_options",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "allow_soft_placement",
            b"allow_soft_placement",
            "cluster_def",
            b"cluster_def",
            "device_count",
            b"device_count",
            "device_filters",
            b"device_filters",
            "experimental",
            b"experimental",
            "gpu_options",
            b"gpu_options",
            "graph_options",
            b"graph_options",
            "inter_op_parallelism_threads",
            b"inter_op_parallelism_threads",
            "intra_op_parallelism_threads",
            b"intra_op_parallelism_threads",
            "isolate_session_state",
            b"isolate_session_state",
            "log_device_placement",
            b"log_device_placement",
            "operation_timeout_in_ms",
            b"operation_timeout_in_ms",
            "placement_period",
            b"placement_period",
            "pluggable_device_options",
            b"pluggable_device_options",
            "rpc_options",
            b"rpc_options",
            "session_inter_op_thread_pool",
            b"session_inter_op_thread_pool",
            "share_cluster_devices_in_session",
            b"share_cluster_devices_in_session",
            "use_per_session_threads",
            b"use_per_session_threads",
        ],
    ) -> None: ...

global___ConfigProto = ConfigProto

@typing.final
class RunOptions(google.protobuf.message.Message):
    """Options for a single Run() call."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _TraceLevel:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _TraceLevelEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[RunOptions._TraceLevel.ValueType], builtins.type
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        NO_TRACE: RunOptions._TraceLevel.ValueType  # 0
        SOFTWARE_TRACE: RunOptions._TraceLevel.ValueType  # 1
        HARDWARE_TRACE: RunOptions._TraceLevel.ValueType  # 2
        FULL_TRACE: RunOptions._TraceLevel.ValueType  # 3

    class TraceLevel(_TraceLevel, metaclass=_TraceLevelEnumTypeWrapper):
        """TODO(pbar) Turn this into a TraceOptions proto which allows
        tracing to be controlled in a more orthogonal manner?
        """

    NO_TRACE: RunOptions.TraceLevel.ValueType  # 0
    SOFTWARE_TRACE: RunOptions.TraceLevel.ValueType  # 1
    HARDWARE_TRACE: RunOptions.TraceLevel.ValueType  # 2
    FULL_TRACE: RunOptions.TraceLevel.ValueType  # 3

    @typing.final
    class Experimental(google.protobuf.message.Message):
        """Everything inside Experimental is subject to change and is not subject
        to API stability guarantees in
        https://www.tensorflow.org/guide/version_compat.
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        @typing.final
        class RunHandlerPoolOptions(google.protobuf.message.Message):
            """Options for run handler thread pool."""

            DESCRIPTOR: google.protobuf.descriptor.Descriptor

            PRIORITY_FIELD_NUMBER: builtins.int
            priority: builtins.int
            """Priority of the request. The run handler thread pool will schedule ops
            based on the priority number. The larger number means higher priority.
            """
            def __init__(self, *, priority: builtins.int | None = ...) -> None: ...
            def ClearField(self, field_name: typing.Literal["priority", b"priority"]) -> None: ...

        COLLECTIVE_GRAPH_KEY_FIELD_NUMBER: builtins.int
        USE_RUN_HANDLER_POOL_FIELD_NUMBER: builtins.int
        RUN_HANDLER_POOL_OPTIONS_FIELD_NUMBER: builtins.int
        collective_graph_key: builtins.int
        """If non-zero, declares that this graph is going to use collective
        ops and must synchronize step_ids with any other graph with this
        same group_key value (in a distributed computation where tasks
        run disjoint graphs).
        """
        use_run_handler_pool: builtins.bool
        """If true, then operations (using the inter-op pool) across all
        session::run() calls will be centrally scheduled, optimizing for (median
        and tail) latency.
        Consider using this option for CPU-bound workloads like inference.
        """
        @property
        def run_handler_pool_options(self) -> global___RunOptions.Experimental.RunHandlerPoolOptions: ...
        def __init__(
            self,
            *,
            collective_graph_key: builtins.int | None = ...,
            use_run_handler_pool: builtins.bool | None = ...,
            run_handler_pool_options: global___RunOptions.Experimental.RunHandlerPoolOptions | None = ...,
        ) -> None: ...
        def HasField(
            self, field_name: typing.Literal["run_handler_pool_options", b"run_handler_pool_options"]
        ) -> builtins.bool: ...
        def ClearField(
            self,
            field_name: typing.Literal[
                "collective_graph_key",
                b"collective_graph_key",
                "run_handler_pool_options",
                b"run_handler_pool_options",
                "use_run_handler_pool",
                b"use_run_handler_pool",
            ],
        ) -> None: ...

    TRACE_LEVEL_FIELD_NUMBER: builtins.int
    TIMEOUT_IN_MS_FIELD_NUMBER: builtins.int
    INTER_OP_THREAD_POOL_FIELD_NUMBER: builtins.int
    OUTPUT_PARTITION_GRAPHS_FIELD_NUMBER: builtins.int
    DEBUG_OPTIONS_FIELD_NUMBER: builtins.int
    REPORT_TENSOR_ALLOCATIONS_UPON_OOM_FIELD_NUMBER: builtins.int
    EXPERIMENTAL_FIELD_NUMBER: builtins.int
    trace_level: global___RunOptions.TraceLevel.ValueType
    timeout_in_ms: builtins.int
    """Time to wait for operation to complete in milliseconds."""
    inter_op_thread_pool: builtins.int
    """The thread pool to use, if session_inter_op_thread_pool is configured.
    To use the caller thread set this to -1 - this uses the caller thread
    to execute Session::Run() and thus avoids a context switch. Using the
    caller thread to execute Session::Run() should be done ONLY for simple
    graphs, where the overhead of an additional context switch is
    comparable with the overhead of Session::Run().
    """
    output_partition_graphs: builtins.bool
    """Whether the partition graph(s) executed by the executor(s) should be
    outputted via RunMetadata.
    """
    report_tensor_allocations_upon_oom: builtins.bool
    """When enabled, causes tensor allocation information to be included in
    the error message when the Run() call fails because the allocator ran
    out of memory (OOM).

    Enabling this option can slow down the Run() call.
    """
    @property
    def debug_options(self) -> tensorflow.core.protobuf.debug_pb2.DebugOptions:
        """EXPERIMENTAL.  Options used to initialize DebuggerState, if enabled."""

    @property
    def experimental(self) -> global___RunOptions.Experimental: ...
    def __init__(
        self,
        *,
        trace_level: global___RunOptions.TraceLevel.ValueType | None = ...,
        timeout_in_ms: builtins.int | None = ...,
        inter_op_thread_pool: builtins.int | None = ...,
        output_partition_graphs: builtins.bool | None = ...,
        debug_options: tensorflow.core.protobuf.debug_pb2.DebugOptions | None = ...,
        report_tensor_allocations_upon_oom: builtins.bool | None = ...,
        experimental: global___RunOptions.Experimental | None = ...,
    ) -> None: ...
    def HasField(
        self, field_name: typing.Literal["debug_options", b"debug_options", "experimental", b"experimental"]
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "debug_options",
            b"debug_options",
            "experimental",
            b"experimental",
            "inter_op_thread_pool",
            b"inter_op_thread_pool",
            "output_partition_graphs",
            b"output_partition_graphs",
            "report_tensor_allocations_upon_oom",
            b"report_tensor_allocations_upon_oom",
            "timeout_in_ms",
            b"timeout_in_ms",
            "trace_level",
            b"trace_level",
        ],
    ) -> None: ...

global___RunOptions = RunOptions

@typing.final
class RunMetadata(google.protobuf.message.Message):
    """Metadata output (i.e., non-Tensor) for a single Run() call."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class FunctionGraphs(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        PARTITION_GRAPHS_FIELD_NUMBER: builtins.int
        PRE_OPTIMIZATION_GRAPH_FIELD_NUMBER: builtins.int
        POST_OPTIMIZATION_GRAPH_FIELD_NUMBER: builtins.int
        @property
        def partition_graphs(
            self,
        ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[tensorflow.core.framework.graph_pb2.GraphDef]:
            """TODO(nareshmodi): Include some sort of function/cache-key identifier?"""

        @property
        def pre_optimization_graph(self) -> tensorflow.core.framework.graph_pb2.GraphDef: ...
        @property
        def post_optimization_graph(self) -> tensorflow.core.framework.graph_pb2.GraphDef: ...
        def __init__(
            self,
            *,
            partition_graphs: collections.abc.Iterable[tensorflow.core.framework.graph_pb2.GraphDef] | None = ...,
            pre_optimization_graph: tensorflow.core.framework.graph_pb2.GraphDef | None = ...,
            post_optimization_graph: tensorflow.core.framework.graph_pb2.GraphDef | None = ...,
        ) -> None: ...
        def HasField(
            self,
            field_name: typing.Literal[
                "post_optimization_graph", b"post_optimization_graph", "pre_optimization_graph", b"pre_optimization_graph"
            ],
        ) -> builtins.bool: ...
        def ClearField(
            self,
            field_name: typing.Literal[
                "partition_graphs",
                b"partition_graphs",
                "post_optimization_graph",
                b"post_optimization_graph",
                "pre_optimization_graph",
                b"pre_optimization_graph",
            ],
        ) -> None: ...

    STEP_STATS_FIELD_NUMBER: builtins.int
    COST_GRAPH_FIELD_NUMBER: builtins.int
    PARTITION_GRAPHS_FIELD_NUMBER: builtins.int
    FUNCTION_GRAPHS_FIELD_NUMBER: builtins.int
    SESSION_METADATA_FIELD_NUMBER: builtins.int
    @property
    def step_stats(self) -> tensorflow.core.framework.step_stats_pb2.StepStats:
        """Statistics traced for this step. Populated if tracing is turned on via the
        "RunOptions" proto.
        EXPERIMENTAL: The format and set of events may change in future versions.
        """

    @property
    def cost_graph(self) -> tensorflow.core.framework.cost_graph_pb2.CostGraphDef:
        """The cost graph for the computation defined by the run call."""

    @property
    def partition_graphs(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[tensorflow.core.framework.graph_pb2.GraphDef]:
        """Graphs of the partitions executed by executors."""

    @property
    def function_graphs(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___RunMetadata.FunctionGraphs]:
        """This is only populated for graphs that are run as functions in TensorFlow
        V2. There will be an entry below for each function that is traced.
        The main use cases of the post_optimization_graph and the partition_graphs
        is to give the caller insight into the graphs that were actually run by the
        runtime. Additional information (such as those in step_stats) will match
        these graphs.
        We also include the pre_optimization_graph since it is usually easier to
        read, and is helpful in situations where the caller wants to get a high
        level idea of what the built graph looks like (since the various graph
        optimization passes might change the structure of the graph significantly).
        """

    @property
    def session_metadata(self) -> global___SessionMetadata:
        """Metadata about the session."""

    def __init__(
        self,
        *,
        step_stats: tensorflow.core.framework.step_stats_pb2.StepStats | None = ...,
        cost_graph: tensorflow.core.framework.cost_graph_pb2.CostGraphDef | None = ...,
        partition_graphs: collections.abc.Iterable[tensorflow.core.framework.graph_pb2.GraphDef] | None = ...,
        function_graphs: collections.abc.Iterable[global___RunMetadata.FunctionGraphs] | None = ...,
        session_metadata: global___SessionMetadata | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "cost_graph", b"cost_graph", "session_metadata", b"session_metadata", "step_stats", b"step_stats"
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "cost_graph",
            b"cost_graph",
            "function_graphs",
            b"function_graphs",
            "partition_graphs",
            b"partition_graphs",
            "session_metadata",
            b"session_metadata",
            "step_stats",
            b"step_stats",
        ],
    ) -> None: ...

global___RunMetadata = RunMetadata

@typing.final
class TensorConnection(google.protobuf.message.Message):
    """Defines a connection between two tensors in a `GraphDef`."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FROM_TENSOR_FIELD_NUMBER: builtins.int
    TO_TENSOR_FIELD_NUMBER: builtins.int
    from_tensor: builtins.str
    """A tensor name. The value of this tensor will be substituted for
    the tensor named in `to_tensor`.
    """
    to_tensor: builtins.str
    """A tensor name. The value of this tensor will be bound to the
    value of the tensor named in `from_tensor`.
    """
    def __init__(self, *, from_tensor: builtins.str | None = ..., to_tensor: builtins.str | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["from_tensor", b"from_tensor", "to_tensor", b"to_tensor"]) -> None: ...

global___TensorConnection = TensorConnection

@typing.final
class CallableOptions(google.protobuf.message.Message):
    """Defines a subgraph in another `GraphDef` as a set of feed points and nodes
    to be fetched or executed.

    Compare with the arguments to `Session::Run()`.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class FeedDevicesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(self, *, key: builtins.str | None = ..., value: builtins.str | None = ...) -> None: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    @typing.final
    class FetchDevicesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(self, *, key: builtins.str | None = ..., value: builtins.str | None = ...) -> None: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    FEED_FIELD_NUMBER: builtins.int
    FETCH_FIELD_NUMBER: builtins.int
    TARGET_FIELD_NUMBER: builtins.int
    RUN_OPTIONS_FIELD_NUMBER: builtins.int
    TENSOR_CONNECTION_FIELD_NUMBER: builtins.int
    FEED_DEVICES_FIELD_NUMBER: builtins.int
    FETCH_DEVICES_FIELD_NUMBER: builtins.int
    FETCH_SKIP_SYNC_FIELD_NUMBER: builtins.int
    fetch_skip_sync: builtins.bool
    """By default, RunCallable() will synchronize the GPU stream before returning
    fetched tensors on a GPU device, to ensure that the values in those tensors
    have been produced. This simplifies interacting with the tensors, but
    potentially incurs a performance hit.

    If this options is set to true, the caller is responsible for ensuring
    that the values in the fetched tensors have been produced before they are
    used. The caller can do this by invoking `Device::Sync()` on the underlying
    device(s), or by feeding the tensors back to the same Session using
    `feed_devices` with the same corresponding device name.
    """
    @property
    def feed(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """Tensors to be fed in the callable. Each feed is the name of a tensor."""

    @property
    def fetch(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """Fetches. A list of tensor names. The caller of the callable expects a
        tensor to be returned for each fetch[i] (see RunStepResponse.tensor). The
        order of specified fetches does not change the execution order.
        """

    @property
    def target(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """Target Nodes. A list of node names. The named nodes will be run by the
        callable but their outputs will not be returned.
        """

    @property
    def run_options(self) -> global___RunOptions:
        """Options that will be applied to each run."""

    @property
    def tensor_connection(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___TensorConnection]:
        """Tensors to be connected in the callable. Each TensorConnection denotes
        a pair of tensors in the graph, between which an edge will be created
        in the callable.
        """

    @property
    def feed_devices(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]:
        """The Tensor objects fed in the callable and fetched from the callable
        are expected to be backed by host (CPU) memory by default.

        The options below allow changing that - feeding tensors backed by
        device memory, or returning tensors that are backed by device memory.

        The maps below map the name of a feed/fetch tensor (which appears in
        'feed' or 'fetch' fields above), to the fully qualified name of the device
        owning the memory backing the contents of the tensor.

        For example, creating a callable with the following options:

        CallableOptions {
          feed: "a:0"
          feed: "b:0"

          fetch: "x:0"
          fetch: "y:0"

          feed_devices: {
            "a:0": "/job:localhost/replica:0/task:0/device:GPU:0"
          }

          fetch_devices: {
            "y:0": "/job:localhost/replica:0/task:0/device:GPU:0"
         }
        }

        means that the Callable expects:
        - The first argument ("a:0") is a Tensor backed by GPU memory.
        - The second argument ("b:0") is a Tensor backed by host memory.
        and of its return values:
        - The first output ("x:0") will be backed by host memory.
        - The second output ("y:0") will be backed by GPU memory.

        FEEDS:
        It is the responsibility of the caller to ensure that the memory of the fed
        tensors will be correctly initialized and synchronized before it is
        accessed by operations executed during the call to Session::RunCallable().

        This is typically ensured by using the TensorFlow memory allocators
        (Device::GetAllocator()) to create the Tensor to be fed.

        Alternatively, for CUDA-enabled GPU devices, this typically means that the
        operation that produced the contents of the tensor has completed, i.e., the
        CUDA stream has been synchronized (e.g., via cuCtxSynchronize() or
        cuStreamSynchronize()).
        """

    @property
    def fetch_devices(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    def __init__(
        self,
        *,
        feed: collections.abc.Iterable[builtins.str] | None = ...,
        fetch: collections.abc.Iterable[builtins.str] | None = ...,
        target: collections.abc.Iterable[builtins.str] | None = ...,
        run_options: global___RunOptions | None = ...,
        tensor_connection: collections.abc.Iterable[global___TensorConnection] | None = ...,
        feed_devices: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        fetch_devices: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        fetch_skip_sync: builtins.bool | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["run_options", b"run_options"]) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "feed",
            b"feed",
            "feed_devices",
            b"feed_devices",
            "fetch",
            b"fetch",
            "fetch_devices",
            b"fetch_devices",
            "fetch_skip_sync",
            b"fetch_skip_sync",
            "run_options",
            b"run_options",
            "target",
            b"target",
            "tensor_connection",
            b"tensor_connection",
        ],
    ) -> None: ...

global___CallableOptions = CallableOptions



import datetime
from collections.abc import Awaitable
from typing import Any

from asgiref.typing import ASGIReceiveCallable, ASGISendCallable
from channels.consumer import _ChannelScope
from channels.utils import _ChannelApplication
from django.contrib.sessions.backends.base import SessionBase

class CookieMiddleware:
    inner: _ChannelApplication

    def __init__(self, inner: _ChannelApplication) -> None: ...

    # Returns the same type as the provided _ChannelApplication.
    async def __call__(self, scope: _ChannelScope, receive: ASGIReceiveCallable, send: ASGISendCallable) -> Any: ...
    @classmethod
    def set_cookie(
        cls,
        message: dict[str, Any],
        key: str,
        value: str = "",
        max_age: int | None = None,
        expires: str | datetime.datetime | None = None,
        path: str = "/",
        domain: str | None = None,
        secure: bool = False,
        httponly: bool = False,
        samesite: str = "lax",
    ) -> None: ...
    @classmethod
    def delete_cookie(cls, message: dict[str, Any], key: str, path: str = "/", domain: str | None = None) -> None: ...

class InstanceSessionWrapper:
    save_message_types: list[str]
    cookie_response_message_types: list[str]
    cookie_name: str
    session_store: SessionBase
    scope: _ChannelScope
    activated: bool
    real_send: ASGISendCallable

    def __init__(self, scope: _ChannelScope, send: ASGISendCallable) -> None: ...
    async def resolve_session(self) -> None: ...
    async def send(self, message: dict[str, Any]) -> Awaitable[None]: ...
    async def save_session(self) -> None: ...

class SessionMiddleware:
    inner: _ChannelApplication

    def __init__(self, inner: _ChannelApplication) -> None: ...
    async def __call__(
        self, scope: _ChannelScope, receive: ASGIReceiveCallable, send: ASGISendCallable
    ) -> _ChannelApplication: ...

def SessionMiddlewareStack(inner: _ChannelApplication) -> _ChannelApplication: ...



from _typeshed import Incomplete, Unused
from collections.abc import Iterable, Sequence
from typing import ClassVar, Final, NoReturn

from docutils import nodes
from docutils.transforms import Transform

__docformat__: Final = "reStructuredText"

class SectNum(Transform):
    default_priority: ClassVar[int]
    maxdepth: int
    startvalue: int
    prefix: str
    suffix: str
    def apply(self) -> None: ...
    def update_section_numbers(self, node: nodes.Element, prefix: Iterable[str] = (), depth: int = 0) -> None: ...

class Contents(Transform):
    default_priority: ClassVar[int]
    toc_id: Incomplete
    backlinks: Incomplete
    def apply(self) -> None: ...
    def build_contents(
        self, node: nodes.Element, level: int = 0
    ) -> nodes.bullet_list | list[None]: ...  # return empty list if entries is empty
    def copy_and_filter(self, node: nodes.Node) -> Sequence[nodes.Node]: ...

class ContentsFilter(nodes.TreeCopyVisitor):
    def get_entry_text(self) -> Sequence[nodes.Node]: ...
    def ignore_node_but_process_children(self, node: Unused) -> NoReturn: ...
    visit_problematic = ignore_node_but_process_children
    visit_reference = ignore_node_but_process_children
    visit_target = ignore_node_but_process_children



from fanstatic.compiler import Compiler as Compiler, Minifier as Minifier, sdist_compile as sdist_compile
from fanstatic.core import (
    BUNDLE_PREFIX as BUNDLE_PREFIX,
    DEBUG as DEBUG,
    DEFAULT_SIGNATURE as DEFAULT_SIGNATURE,
    MINIFIED as MINIFIED,
    NEEDED as NEEDED,
    VERSION_PREFIX as VERSION_PREFIX,
    ConfigurationError as ConfigurationError,
    Group as Group,
    GroupResource as GroupResource,
    Library as Library,
    LibraryDependencyCycleError as LibraryDependencyCycleError,
    NeededResources as NeededResources,
    Resource as Resource,
    Slot as Slot,
    SlotError as SlotError,
    UnknownResourceError as UnknownResourceError,
    UnknownResourceExtension as UnknownResourceExtension,
    UnknownResourceExtensionError as UnknownResourceExtensionError,
    clear_needed as clear_needed,
    del_needed as del_needed,
    get_needed as get_needed,
    init_needed as init_needed,
    register_inclusion_renderer as register_inclusion_renderer,
    set_auto_register_library as set_auto_register_library,
    set_resource_file_existence_checking as set_resource_file_existence_checking,
)
from fanstatic.inclusion import Inclusion as Inclusion, bundle_resources as bundle_resources, sort_resources as sort_resources
from fanstatic.injector import Injector as Injector, make_injector as make_injector
from fanstatic.publisher import (
    Delegator as Delegator,
    LibraryPublisher as LibraryPublisher,
    Publisher as Publisher,
    make_publisher as make_publisher,
)
from fanstatic.registry import (
    CompilerRegistry as CompilerRegistry,
    LibraryRegistry as LibraryRegistry,
    MinifierRegistry as MinifierRegistry,
    get_library_registry as get_library_registry,
)
from fanstatic.wsgi import Fanstatic as Fanstatic, Serf as Serf, make_fanstatic as make_fanstatic, make_serf as make_serf



from abc import ABCMeta
from collections.abc import ItemsView, Mapping, ValuesView
from io import StringIO as StringIO
from re import Pattern
from typing import Any, Final, SupportsIndex, TypeVar
from typing_extensions import TypeGuard
from urllib.parse import parse_qs, quote, unquote, urlencode as urlencode, urlparse as urlparse

_KT = TypeVar("_KT")
_VT_co = TypeVar("_VT_co", covariant=True)

url_quote = quote
url_unquote = unquote
url_parse_qs = parse_qs

PY2: Final = False
PY3: Final = True
RE_NUM: Final[Pattern[str]]
ON_LINUX: Final[bool]
ON_OSX: Final[bool]
ON_WINDOWS: Final[bool]

class AbstractBase(metaclass=ABCMeta): ...

SOCKET_ERROR = OSError
SOL_TCP: Final[int]
basestring: Final[tuple[type[str]]]
str_or_bytes: Final[tuple[type[str], type[bytes]]]
xrange = range
unicode_type = str

def time_now() -> float: ...
def dictkeys(dct: Mapping[_KT, Any]) -> list[_KT]: ...
def dictvalues(dct: Mapping[Any, _VT_co]) -> list[_VT_co]: ...
def dict_iteritems(dct: Mapping[_KT, _VT_co]) -> ItemsView[_KT, _VT_co]: ...
def dict_itervalues(dct: Mapping[Any, _VT_co]) -> ValuesView[_VT_co]: ...
def byte(*args: SupportsIndex) -> bytes: ...

class long(int): ...

def canonical_str(value: object) -> str: ...
def is_integer(value: object) -> TypeGuard[int]: ...
def as_bytes(value: str | bytes) -> bytes: ...
def to_digit(value: str) -> int: ...
def get_linux_version(release_str: str) -> tuple[int, int, int]: ...

HAVE_SIGNAL: Final[bool]
EINTR_IS_EXPOSED: Final = False
LINUX_VERSION: tuple[int, int, int] | None



from logging import Logger

from authlib.oauth2 import OAuth2Request
from authlib.oauth2.rfc6749 import BaseGrant
from authlib.oidc.core import UserInfo

log: Logger

class OpenIDToken:
    def get_jwt_config(self, grant: BaseGrant) -> dict[str, str | int]: ...
    def generate_user_info(self, user, scope: str) -> UserInfo: ...
    def get_audiences(self, request: OAuth2Request) -> list[str]: ...
    def process_token(self, grant: BaseGrant, response) -> dict[str, str | int]: ...
    def __call__(self, grant: BaseGrant) -> None: ...

class OpenIDCode(OpenIDToken):
    require_nonce: bool
    def __init__(self, require_nonce: bool = False) -> None: ...
    def exists_nonce(self, nonce: str, request: OAuth2Request) -> bool: ...
    def validate_openid_authorization_request(self, grant: BaseGrant, redirect_uri) -> None: ...
    def __call__(self, grant: BaseGrant) -> None: ...
